/**
 * Project mosaic
 *
 * AbstractModelComponent.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import mosaic.sim.*; 
import mosaic.sim.kernel.control.*;
import java.util.*;
import java.awt.*;

/**
* Abstract superclass for all classes implementing model components.
* <p>
* Implements the interface of ModelComponent.
* <p>
* AbstractModelComponent is a basic implementation of the ModelComponent interface.
* All components in the simulation system should extend directly or indirectly AbstractModelComponent.
* <p>
* Currently ModelComponent extends javax.swing.JComponent to allow interactive building in IDE.
* This will be deleted in the final version.
* <p>
* @see ModelComponent
*/
abstract public class AbstractModelComponent extends javax.swing.JComponent implements ModelComponent {


// Constructors ------------------------------------------------------------------------------------

    /** Default constructor AbstractModelComponent */
    public AbstractModelComponent() {
        this("modelComponent");
    }

    /** Constructor for AbstractModelComponent with initial label*/
    public AbstractModelComponent(String label) {
        setLabel (label);
    }


    /** Constructor for AbstractModelComponent with initial parent*/
/*delete    public AbstractModelComponent(ModelComponent parent) {
        this(null, "modelComponent");
    }
*/

    /** Constructor for AbstractModelComponent with initial parent and label*/
/*delete    public AbstractModelComponent(ModelComponent parent, String label) {
        setLabel (label);
    }
*/

// reset -------------------------------------------------------------------------------------------

	/** Resets the model component by calling intialize */
    public void reset() {
        initialize();
    }

// initialize --------------------------------------------------------------------------------------

    /** Initializes the the model component. */
    public void initialize() {
    }

// simulation time ---------------------------------------------------------------------------------

    /**
     * Gets the current simulation time of the simulation application this component belongs to.
     * @return       the current simulation time as a double value
     */
    public double getTime() {
        if (getSimApplication() != null) {
            return  getSimApplication().getTime();
        } else {
            return 0.0;
        }
    }

// simulation application --------------------------------------------------------------------------

    private SimApplication application;

    /**
     * Gets the simulation application object this component belongs to
     * @ return       the simulation application object
     */
    public SimApplication getSimApplication() {
        if (application != null) {
            return application;
        } else {
            return SimApplication.getCurrentSimApplication();
        }
    }

    /**
     * Sets the simulation application object this component belongs to
     * @param appl the simulation application object
     */
    public void setSimApplication(SimApplication appl) {
        application = appl;
        Iterator i = getModelComponents().iterator();
        while (i.hasNext()) {
            ((ModelComponent) i.next()).setSimApplication(application);
        }
    }


// enabled -----------------------------------------------------------------------------------------

    /** Boolean to hold the enabled status of this model element */
    protected boolean enabled = true;

    /** Test if this model element is enabled */
    public boolean isEnabled() {
        return enabled;
    }

    /** Test if this model element is not disabled */
    public boolean isDisabled () {
        return (! enabled);
    }

    /** Sets the enabled status of the model element */
    public void setEnabled (boolean _enabled) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
        }
        enabled = _enabled;
        if (enabled){
            fireModelComponentEnabled();
        }else{
            fireModelComponentDisabled();
        }
    }

    /** Enable the model component */
    public void enable () {
        setEnabled(true);
    }

    /** Disable the model component */
    public void disable () {
        setEnabled(false);
    }


// label -------------------------------------------------------------------------------------------

    /** variable to hold the label of the component. */
    protected String label;

    /*
     * Gets the label of the component.
     * @return        the label of the component
     */
    public String getLabel() {
        return label;
    }

    /**
     * Sets the label of the component.
     * @param label      the label for the component
     */
    public  void setLabel(String label) {
        this.label = label;
    }

    /**
     * Generates an information string for the component.
     * @return        the information string
     */
    public String getInfo() {
        String text = getClass().getName() + ": " + getLabel();
        return text;
    }

// model components --------------------------------------------------------------------------------


    /** Collection to contain the subcomponents for this component. */
    protected java.util.List components = new ArrayList();

    /**
     * Adds a component to this component.
     * @param component   the ModelComponent to be added to this component
     */
    public boolean addModelComponent(ModelComponent component) {
        // do nothing in test for state events
        if (getSimApplication().getIntegrator().testForStateEvents) {
            return false;
        }
        if (! components.contains(component)) {
            components.add(component);
            component.setParentModelComponent (this);
            component.setSimApplication(getSimApplication());
            fireModelComponentAdded(component);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Removes a component from this component.
     * @param component   the ModelComponent to be removed from this component
     */
    public boolean removeModelComponent(ModelComponent component) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            return false ;
        } else if (components.contains(component)) {
            components.remove(component);
            component.setParentModelComponent (null);
            component.setSimApplication(null);
            fireModelComponentRemoved(component);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Gets a collection of all components.
     * @return        a collection containing all sub-component of this component
     */
    public Collection getModelComponents () {
        return components;
    }

    /**
     * Tests if the component is contained in this ModelComponent.
     * @return        true if the component is contained in this component
     */
    public boolean containsModelComponent (ModelComponent component) {
        return (components.contains(component));
    }


// parent model component --------------------------------------------------------------------------

    /** variable to hold the parent of this component */
    protected ModelComponent parentModelComponent = null;

    /**
     * Gets the parent ModelComponent.
     * @return        the ModelComponent which is the parent of this component
     */
    public ModelComponent getParentModelComponent () {
        return parentModelComponent;
    }

    /**
     * Sets the parent for this component.
     * @param parent   the ModelComponent which is the parent of this component
     */
    public void setParentModelComponent(ModelComponent parentModelComponent) {
        this.parentModelComponent = parentModelComponent;
    }

    /**
     * Finds the superior container which is a ModelComponent
     * @return the parent ModelComponent
     */
    public ModelComponent findParentModelComponent () {
        Container _parent = getParent();
        while (_parent != null && ! (_parent instanceof ModelComponent)) {
            _parent = _parent.getParent();
        }
        if (_parent != null &&  (_parent instanceof ModelComponent))
            return ((ModelComponent) _parent);
        else
            return null;
    }


// ModelComponentEvent -----------------------------------------------------------------------------

    /** the support object to handle model component events. */
    protected ModelComponentEventSupport sCESupport = new ModelComponentEventSupport();

    /**
    * Adds a ModelComponentEventListener to this component.
    * @param listener   the listener to be added
    */
    public void addModelComponentEventListener(ModelComponentEventListener listener) {
         sCESupport.addListenerAllEvents(listener);
    }

    /**
    * Removes a ModelComponentEventListener from this component.
    * @param listener   the listener to be removed
    */
    public void removeModelComponentEventListener(ModelComponentEventListener listener) {
         sCESupport.removeListenerAllEvents(listener);
    }


    /**
    * Adds ModelComponentEventListener for event <code>eventName</code> to this component.
    * @param eventName  the name of the event
    * @param listener   the listener to be added
    */
    public void addModelComponentEventListener(String eventName, ModelComponentEventListener listener) {
         sCESupport.addListener(eventName, listener);
    }

    /**
    * Removes ModelComponentEventListener for event <code>eventName</code> from this component.
    * @param eventName  the name of the event
    * @param listener   the listener to be removed
    */
    public void removeModelComponentEventListener(String eventName, ModelComponentEventListener listener) {
         sCESupport.removeListener(eventName, listener);
    }

    /**
     * Fires an addition event for this model component
     * @param element the element which has been added
     */
    public void fireModelComponentAdded (ModelComponent element) {
        sCESupport.fireAddedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_ADDED, element));
    }

     /**
     * Fires a removal event for this model component.
     * @param element the element which has been removed
     */
    public void fireModelComponentRemoved (ModelComponent element) {
        sCESupport.fireRemovedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_REMOVED, element));
    }

    /**
     * Fires an enabled event for this model component.
     */
    public void fireModelComponentEnabled () {
        sCESupport.fireEnabledEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_ENABLED));
    }

    /**
     * Fires a disabled event for this model component.
     */
    public void fireModelComponentDisabled () {
        sCESupport.fireDisabledEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_DISABLED));
    }

    /**
     * Fires an event when this model component got continuous.
     */
    public void fireModelComponentGotContinuous () {
        sCESupport.fireGotContinuousEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_GOT_CONTINUOUS));
    }

    /**
     * Fires an event when this model component got discrete.
     */
    public void fireModelComponentGotDiscrete () {
        sCESupport.fireGotDiscreteEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_GOT_DISCRETE));
    }

    /**
     * Fires an event for this model component when it has changed.
     */
    public void fireModelComponentChanged () {
        sCESupport.fireChangedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_CHANGED));
    }


    /**
     * Fires an event for this model component when it has changed.
     * @param e The object describing the event.
     */
    public void fireModelComponentChanged (ModelComponentEvent e) {
        sCESupport.fireChangedEvent(e);
    }


// log output --------------------------------------------------------------------------------------

    /** Prints a message on the default log stream */
    final public void printLogOutput(String message) {
        getSimApplication().logOutput(message);
    }

    /** Prints a message of type messageType on the log stream */
    final public void printLogOutput(String messageType, String message) {
        getSimApplication().logOutput(messageType, message);
    }

    /** Prints a error message on the default log stream */
    final public void printErrorOutput(String message) {
        getSimApplication().errorOutput(message);
    }

    /** Prints a error message of type messageType on the log stream */
    final public void printErrorOutput(String messageType, String message) {
        getSimApplication().errorOutput(messageType, message);
    }

    /** Prints a debug message on the log stream */
    final public void printDebugOutput(String message) {
        getSimApplication().debugOutput(message);
    }

    /** Prints a debug message of type messageType on the log stream */
    final public void printDebugOutput(String messageType, String message) {
        getSimApplication().debugOutput(messageType, message);
    }

    /** Prints a debug message on the default log stream */
    final public void printDebugOutput(String message, int debugLevel) {
        getSimApplication().debugOutput(message, debugLevel);
    }

    /** Prints a debug message of type messageType on the log stream */
    final public void printDebugOutput(String messageType, String message, int debugLevel) {
        getSimApplication().debugOutput(messageType, message, debugLevel);
    }


// -------------------------------------------------------------------------------------------------
// Component implementation
// -------------------------------------------------------------------------------------------------


    public Dimension getMinimumSize() {
        return new Dimension(10,10);
    }

    public Dimension getPreferredSize() {
        return new Dimension(30,20);
    }

    /**
     * Adds the component to the parent ModelComponent when added into the JComponent hierarchy.
     * Specialized from JComponent.
     */
     public void addNotify() {
        super.addNotify();
        ModelComponent parentModelComp = findParentModelComponent();
        if (parentModelComp != null) {
            parentModelComp.addModelComponent(this);
        }
     }

    /**
     * Removes the component to the parent ModelComponent when removed from the JComponent hierarchy.
     * Specialized from JComponent.
     */
     public void removeNotify() {
        super.removeNotify();
        if (getParentModelComponent() != null) {
            getParentModelComponent().removeModelComponent(this);
        }
    }

    /**
     * Paints the model by drawing a rectangle and the label of the model.
     * param g the graphics context to paint on
     */
    public void paintComponent (Graphics g) {
        int h = getSize().height;
        int w = getSize().width;
        g.setColor(getForeground());
        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        String l = getLabel();
        if (l == null) {
            l = getClass().getName();
            int i = label.lastIndexOf('.');
            label = label.substring(i+1);
        }
        g.draw3DRect(0, 0, w-1, h-1, true);
        g.drawString(l,3,h-fm.getMaxDescent()-3);
    }

} // end AbstractModelComponent ====================================================================

