/**
 * Project mosaic
 *
 * ModelComponentEventListener.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

/**
 * This class implements an event support for ModelComponent. It is based on BasicEventSupport.
 * Any major changes should be made there. (see BasicEventSupport)
 */

import java.util.*;
import concurrent.CopyOnWriteArrayList;
import concurrent.SyncMap;
import concurrent.ReentrantWriterPreferenceReadWriteLock;

public class ModelComponentEventSupport {

    static final String MODEL_COMPONENT_ADDED           ="modelComponentAdded";
    static final String MODEL_COMPONENT_REMOVED         ="modelComponentRemoved";
    static final String MODEL_COMPONENT_ENABLED         ="modelComponentEnabled";
    static final String MODEL_COMPONENT_DISABLED        ="modelComponentDisabled";
    static final String MODEL_COMPONENT_GOT_CONTINUOUS  ="modelComponentGotContinuous";
    static final String MODEL_COMPONENT_GOT_DISCRETE    ="modelComponentGotDiscrete";
    static final String MODEL_COMPONENT_CHANGED         ="modelComponentChanged";

    /** Maps the eventName to the list of listeners. */
    protected Map lists=new SyncMap(new HashMap(15),new ReentrantWriterPreferenceReadWriteLock());

    /** Contains the listeners that should get all events. */
    protected List allEvents=new CopyOnWriteArrayList();

    /** Adds the listener to the list for receiving all events. */
    public void addListenerAllEvents(ModelComponentEventListener listener){
        allEvents.add(listener);
    }

    /** Registers a listener for special events. */
    public void addListener(String eventName, ModelComponentEventListener listener){
        List list=(List) lists.get(eventName);
        if (list==null){ // create new list
            list=new CopyOnWriteArrayList();
            list.add(listener);
            lists.put(eventName,list);
        }else{
            list.add(listener);
        }
    }

    /**
     * This method removes the listener from the list of listeners that receive all events. It
     * does NOT remove it from any other list! The removeListener method has to be used
     * instead.
     */
    public void removeListenerAllEvents(ModelComponentEventListener listener){
        allEvents.remove(listener);
    }

    /**
     * This method removes the listener from the list of the given event.
     */
    public void removeListener(String eventName, ModelComponentEventListener listener){
        List list=(List) lists.get(eventName);
        if (list!=null){
            list.remove(listener);
        }
    }

    /** Fire ModelComponent added event. */
    public void fireAddedEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_ADDED);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentAdded(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentAdded(event);
            }
        }
    }

    /** Fire ModelComponent removed event. */
    public void fireRemovedEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_REMOVED);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentRemoved(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentRemoved(event);
            }
        }
    }

    /** Fire ModelComponent enabled event. */
    public void fireEnabledEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_ENABLED);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentEnabled(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                    listener.modelComponentEnabled(event);
            }
        }
    }

    /** Fire ModelComponent disabled event. */
    public void fireDisabledEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_DISABLED);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentDisabled(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                    listener.modelComponentDisabled(event);
            }
        }
    }

    /** Fire ModelComponent disabled event. */
    public void fireGotDiscreteEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_GOT_DISCRETE);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentGotDiscrete(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                    listener.modelComponentGotDiscrete(event);
            }
        }
    }

    /** Fire ModelComponent disabled event. */
    public void fireGotContinuousEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_GOT_CONTINUOUS);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentGotContinuous(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                    listener.modelComponentGotContinuous(event);
            }
        }
    }

    /** Fire ModelComponent changed event. */
    public void fireChangedEvent(ModelComponentEvent event){
        List list=(List) lists.get(MODEL_COMPONENT_CHANGED);
        ModelComponentEventListener listener;
        Iterator i;
        if (list!=null && !list.isEmpty()){
            i=list.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                listener.modelComponentChanged(event);
            }
        }
        if (allEvents.size()>0){
            i=allEvents.iterator();
            while (i.hasNext()){
                listener=(ModelComponentEventListener) i.next();
                    listener.modelComponentChanged(event);
            }
        }
    }
} // end ModelComponentEventSupport ================================================================

