/**
 * Project mosaic
 *
 * ModelPanel.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import mosaic.sim.*; 
import java.util.*;
import java.awt.*;


/**
 * Class to serve as a panel for ModelComponents.
 * Use this class as a superclass to implement a panel which hold
 * model components of your simulation application.
 */
public class ModelPanel  extends javax.swing.JPanel implements ModelComponent {

    /**
    * Initializes the application components.
    */
    private void jbInit() throws Exception {
    }

// -----------------------------------------------------------------------------
// basic implementation of SimulationPanel (do not change) ---------------------
// -----------------------------------------------------------------------------

  /** Constructor setting up the menus and creating the rootModel and the Simulator */
  public ModelPanel () {
  }

// reset -------------------------------------------------------------------------------------------

	/** Resets the model, intializes its state */
    public void reset() {
        initialize();
    }

// initialize --------------------------------------------------------------------------------------

    /** Initializes the state of the model */
    public void initialize() {
    }

// simulation time ---------------------------------------------------------------------------------

    /**
     * Gets the current simulation time for this simulation application
     * @return       the current simulation time as a double value
     */
    public double getTime() {
        if (getSimApplication() != null) {
            return  getSimApplication().getTime();
        } else {
            return 0.0;
        }
    }

// simulation application --------------------------------------------------------------------------

    private SimApplication application;

    /**
     * Gets the simulation application object this component belongs to
     * @ return       the simulation application object
     */
    public SimApplication getSimApplication() {
        if (application != null) {
            return application;
        } else {
            return SimApplication.getCurrentSimApplication();
        }
    }

    /**
     * Sets the simulation application object this component belongs to
     * @param appl the simulation application object
     */
    public void setSimApplication(SimApplication appl) {
        application = appl;
        Iterator i = getModelComponents().iterator();
        while (i.hasNext()) {
            ((ModelComponent) i.next()).setSimApplication(application);
        }
    }


// enabled -----------------------------------------------------------------------------------------

    /** Boolean to hold the enabled status of this model element */
    protected boolean enabled = true;

    /** Test if this model element is enabled */
    public boolean isEnabled() {
        return enabled;
    }

    /** Test if this model element is not enabled */
    public boolean isDisabled () {
        return (! enabled);
    }

    /** Sets the enabled status of the model element */
    public void setEnabled (boolean _enabled) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
        }
        enabled = _enabled;
        if (enabled){
            fireModelComponentEnabled();
        }else{
            fireModelComponentDisabled();
        }
    }

    /** Enable the model component */
    public void enable () {
        setEnabled(true);
    }

    /** Disable the model component */
    public void disable () {
        setEnabled(false);
    }


// label -------------------------------------------------------------------------------------------

    /** variable to hold the label of the component. */
    protected String label;

    /**
     * Sets the label of the component.
     * @param label      the label for the component
     */
    public  void setLabel(String label) {
        this.label = label;
    }

    /*
     * Gets the label of the component.
     * @return        the label of the component
     */
    public String getLabel() {
        return label;
    }

    /**
     * Generates an information string for the component.
     * @return        the information string
     */
    public String getInfo() {
        String text = getClass().getName() + ": " + getLabel();
        return text;
    }

// model components --------------------------------------------------------------------------------


    /** Collection to contain the subcomponents for this component. */
    protected java.util.List components = new ArrayList();

    /**
     * Adds a component to this component.
     * @param component   the ModelComponent to be added to this component
     */
    public boolean addModelComponent(ModelComponent component) {
        // do nothing in test for state events
        if (getSimApplication().getIntegrator().testForStateEvents) {
            return false;
        }
        if (! components.contains(component)) {
            components.add(component);
            component.setParentModelComponent (this);
            component.setSimApplication(getSimApplication());
            fireModelComponentAdded(component);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Removes a component from this component.
     * @param component   the ModelComponent to be removed from this component
     */
    public boolean removeModelComponent(ModelComponent component) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            return false ;
        } else if (components.contains(component)) {
            components.remove(component);
            component.setParentModelComponent (null);
            component.setSimApplication(null);
            fireModelComponentRemoved(component);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Gets a collection of all components.
     * @return        a collection containing all sub-component of this component
     */
    public Collection getModelComponents () {
        return components;
    }

    /**
     * Tests if the component is contained in this ModelComponent.
     * @return        true if the component is contained in this component
     */
    public boolean containsModelComponent (ModelComponent component) {
        return (components.contains(component));
    }


// parent model component --------------------------------------------------------------------------

    /** variable to hold the parent of this component */
    protected ModelComponent parent = null;

    /**
     * Gets the parent ModelComponent.
     * @return        the ModelComponent which is the parent of this component
     */
    public ModelComponent getParentModelComponent () {
        return parent;
    }

    /**
     * Sets the parent for this component.
     * @param parent   the ModelComponent which is the parent of this component
     */
    public void setParentModelComponent(ModelComponent parent) {
        this.parent = parent;
    }

    /**
     * Finds the superior container which is a ModelComponent
     * @return the parent ModelComponent
     */
    public ModelComponent findParentModelComponent () {
        while (parent != null && !(parent instanceof ModelComponent)) {
            parent = parent.getParentModelComponent();
        }
        if (parent != null &&  (parent instanceof ModelComponent))
            return ((ModelComponent) parent);
        else
            return null;
    }


// ModelComponentEvent -----------------------------------------------------------------------------

    protected ModelComponentEventSupport sCESupport = new ModelComponentEventSupport();

    /**
    * Adds ModelComponentEventListener to this component.
    * @param listener   the listener to be added
    */
    public void addModelComponentEventListener(ModelComponentEventListener listener) {
         sCESupport.addListenerAllEvents(listener);
    }

    /**
    * Removes ModelComponentEventListener to this component.
    * @param listener   the listener to be removed
    */
    public void removeModelComponentEventListener(ModelComponentEventListener listener) {
         sCESupport.removeListenerAllEvents(listener);
    }


    /**
    * Adds ModelComponentEventListener to this component.
    * @param eventName  the name of the event
    * @param listener   the listener to be added
    */
    public void addModelComponentEventListener(String eventName, ModelComponentEventListener listener) {
         sCESupport.addListener(eventName, listener);
    }

    /**
    * Removes ModelComponentEventListener to this component.
    * @param eventName  the name of the event
    * @param listener   the listener to be removed
    */
    public void removeModelComponentEventListener(String eventName, ModelComponentEventListener listener) {
         sCESupport.removeListener(eventName, listener);
    }


    public void fireModelComponentAdded (ModelComponent element) {
        sCESupport.fireAddedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_ADDED, element));
    }

    public void fireModelComponentRemoved (ModelComponent element) {
        sCESupport.fireRemovedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_REMOVED, element));
    }

    public void fireModelComponentEnabled () {
        sCESupport.fireEnabledEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_ENABLED));
    }

    public void fireModelComponentDisabled () {
        sCESupport.fireDisabledEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_DISABLED));
    }

    public void fireModelComponentGotContinuous () {
        sCESupport.fireGotContinuousEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_GOT_CONTINUOUS));
    }

    public void fireModelComponentGotDiscrete () {
        sCESupport.fireGotDiscreteEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_GOT_DISCRETE));
    }

    public void fireModelComponentChanged () {
        sCESupport.fireChangedEvent(
            new ModelComponentEvent(this, ModelComponentEvent.MODEL_COMPONENT_CHANGED));
    }


    public void fireModelComponentChanged (ModelComponentEvent e) {
        sCESupport.fireChangedEvent(e);
    }


// log output --------------------------------------------------------------------------------------

    /** Prints a message on the default log stream */
    final public void printLogOutput(String message) {
        getSimApplication().logOutput(message);
    }

    /** Prints a message of type messageType on the log stream */
    final public void printLogOutput(String messageType, String message) {
        getSimApplication().logOutput(messageType, message);
    }

    /** Prints a error message on the default log stream */
    final public void printErrorOutput(String message) {
        getSimApplication().errorOutput(message);
    }

    /** Prints a error message of type messageType on the log stream */
    final public void printErrorOutput(String messageType, String message) {
        getSimApplication().errorOutput(messageType, message);
    }

    /** Prints a error message on the default log stream */
    final public void printDebugOutput(String message) {
        getSimApplication().debugOutput(message);
    }

    /** Prints a error message of type messageType on the log stream */
    final public void printDebugOutput(String messageType, String message) {
        getSimApplication().debugOutput(messageType, message);
    }

    /** Prints a error message on the default log stream */
    final public void printDebugOutput(String message, int debugLevel) {
        getSimApplication().debugOutput(message, debugLevel);
    }

    /** Prints a error message of type messageType on the log stream */
    final public void printDebugOutput(String messageType, String message, int debugLevel) {
        getSimApplication().debugOutput(messageType, message, debugLevel);
    }


// -------------------------------------------------------------------------------------------------
// Component implementation
// -------------------------------------------------------------------------------------------------


    /** Gets the minimum size for the LayoutManager */
    public Dimension getMinimumSize() {
        return new Dimension(40,40);
    }

    /** Gets the preferred size for the LayoutManager */
    public Dimension getPreferredSize() {
        return new Dimension(400,400);
    }


    /**
     * Adds the component to the parent ModelComponent when added into the JComponent hierarchy.
     * Specialized from JComponent.
     */
     public void addNotify() {
        super.addNotify();
        ModelComponent parentModelComp = findParentModelComponent();
        if (parentModelComp != null) {
            parentModelComp.addModelComponent(this);
        }
     }

    /**
     * Removes the component to the parent ModelComponent when removed from the JComponent hierarchy.
     * Specialized from JComponent.
     */
     public void removeNotify() {
        super.removeNotify();
        if (getParentModelComponent() != null) {
            getParentModelComponent().removeModelComponent(this);
        }
    }

} // end AbstractModelComponent ====================================================================


