/**
 * Project mosaic
 *
 * SimApplication.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import java.util.*;
import java.io.*;
import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.continuous.*;
import mosaic.util.*;

//=================================================================================================/

/**
* This is the implementation of a application object for a mosaic simulation program.
* It represents the root object of a mosaic simulation application.
* One object of that type exists for an independent simulation application.
* It holds the global variables like simulation time and provides access to
* the objects responsible to carry out the simulation like simulator, integrator, animator,
* scheduler, log output stream, simulation main frame.
* It also is the root model, i.e, the root of the model component hierarchy.
* <p>
* This class should be specialized to implement your applications.
* @see sim.examples.blank.BlankApplication 
*/
public class SimApplication extends AbstractModelComponent {

    /** Default constructor */
    public SimApplication() {
        currentSimApplication = this;
        init();
    }


// initialize and clean up -------------------------------------------------------------------------

    /** Initialize the Simulation by creating the main objects and resetting the time */
    public void init() {
        logStream = new LogOutput();
        logStream.setLogStream("simulation", System.out);
        logOutput("start initializing simulation application");
        setSimApplication(this);
        createNewIntegrator();
        createNewSimulator();
        createNewAnimator();
        createNewSimulationScheduler();
        options = new SimOptions(this);
        time = new DoubleVariable("time", 0.0);
        getRootModel().addModelComponent(time);
        resetTime();
        logOutput("initializing simulation application completed");
    }

    /** Resets the simulation application and all its components to an initial state */
    public void reset() {
        if (!getSimulationScheduler().isRunning()) {
            super.reset();
            resetTime();
            getSimulator().reset();
            // reset the component hierarchy 
            ResetModelComponentVisitor resetVisitor = new ResetModelComponentVisitor();
            Iterator simElems = getModelComponents().iterator();
            while (simElems.hasNext()) {
                ModelComponent simComp = ((ModelComponent) simElems.next());
                resetVisitor.visit(simComp);
            }
            // fire variable changes for all variables 
            getSimApplication().getSimulator().fireDiscreteAndContinuousVariableChanges();
        } else {
            logOutput("Tried to reset simulation at time " + getTime() +
                        " while simulation is running! Stop simulation first!");
        }
    }


    /** Cleans up this application by deleting the main objects.  */
    public void cleanUp() {
        resetTime();
        deleteSimulator();
        deleteIntegrator();
        deleteAnimator();
        deleteSimulationScheduler();
        options = new SimOptions(this);
    }

// current application needed for initialization ---------------------------------------------------

    /** Holds the simulation application object recently created. */
    static SimApplication currentSimApplication;

    /** Gets the simulation application object recently created. */
    public static SimApplication getCurrentSimApplication() {
        return currentSimApplication;
    }
    

// addModelComponent specialized to do reset when a sim component is added ---------------------------

    /** Add a model component to this application and
     * fires variable changes for all variables within the component.  */
    public boolean addModelComponent(ModelComponent component) {
        boolean success = super.addModelComponent(component);
        getSimulator().fireDiscreteAndContinuousVariableChanges();
        return success;
    }

// stepLock ----------------------------------------------------------------------------------------

	/** Object to be used as a lock for locking the simulation steps */
    public Object stepLock = new Object();


// global simulation time --------------------------------------------------------------------------

    /** variable to hold the current simulation time */
    DoubleVariable time = new DoubleVariable();

    /** Gets the current simulation time.  */
    final public double getTime() {
        if (time != null) {
            return time.getValue();
        } else {
            return 0.0;
        }
    }

    /** Gets the current simulation time.  */
    final public DoubleVariable getTimeVariable() {
        return time;
    }

    /** Sets the current simulation time */
    public void setTime(double newTime) {
        time.setValue(newTime);
    }

    /** Resets the current simulation time to 0 */
    public void resetTime() {
        setTime(0.0);
    }


//singleton root ModelComponent --------------------------------------------------------------------

    /** Gets the single root model object which is this application itself. */
    public ModelComponent getRootModel() {
        return this;
    }

//singleton Simulator ------------------------------------------------------------------------------

    /** variable holding the single simulator object. */
    private Simulator simulator = null;

    /** Gets the single simulator object */
    public Simulator getSimulator() {
        return simulator;
    }

    /** Create the single simulator object */
    protected void createNewSimulator() {
        simulator = new Simulator(this);
        logOutput("new simulator created");
    }

    /** Delete the single simulator object */
    protected void deleteSimulator() {
        simulator = null;
        logOutput("simulator deleted");
    }

//singleton Integrator -----------------------------------------------------------------------------

    /** variable holding the single Integrator object. */
    private Integrator integrator= null;

    /** Gets the single Integrator object */
    public Integrator getIntegrator() {
        if (integrator==null) {
            createNewIntegrator();
        }
        return integrator;
    }

    /** Create the single Integrator object */
    protected void createNewIntegrator() {
        integrator = new Integrator(this);
        logOutput("new integrator created");
    }

    /** Delete the single Integrator object */
    protected void deleteIntegrator() {
        integrator = null;
        logOutput("integrator deleted");
    }

//singleton Animator -------------------------------------------------------------------------------

    /** variable holding the single Animator object. */
    private Animator animator= null;

    /** Gets the single Animator object */
    public Animator getAnimator() {
        if (animator == null) {
            animator = new Animator();
        }
        return animator;
    }

    /** Create the single Animator object */
    protected void createNewAnimator() {
        animator =  new Animator();
        logOutput("new animator created");
    }

    /** Delete the single Animator object */
    protected void deleteAnimator() {
        animator =  null;
        logOutput("animator deleted");
    }

//singleton SimulationScheduler --------------------------------------------------------------------

    /** variable holding the single SimulationScheduler object. */
    private SimulationScheduler simulationScheduler= null;

    /** Gets the single SimulationScheduler object */
    public SimulationScheduler getSimulationScheduler() {
        if ( simulationScheduler == null) {
            simulationScheduler = new SimulationScheduler(this);
        }
        return simulationScheduler;
    }

    /** Create the single SimulationScheduler object */
    protected void createNewSimulationScheduler() {
        simulationScheduler =  new SimulationScheduler(this);
        logOutput("new simulationScheduler created");
    }

    /** Delete the single simulationScheduler object */
    protected void deleteSimulationScheduler() {
        simulationScheduler =  null;
        logOutput("simulationScheduler deleted");
    }

//singleton SimulationMainFrame --------------------------------------------------------------------

    /** variable holding the  main frame object. */
    private SimMainFrame mainFrame = null;

    /** Gets the  main frame object. */
    public SimMainFrame getMainFrame() {
        return mainFrame;
    }

    /** Sets the  main frame object. */
    public void setMainFrame(SimMainFrame newMainFrame) {
        mainFrame = newMainFrame;
        logOutput("main frame set");
    }

    /** Delete the  main frame object.  */
    protected void deleteMainFrame() {
        mainFrame = null;
        logOutput("main deleted");
    }


// log output --------------------------------------------------------------------------------------

    /** variable to hold a log output stream for this application */
    private LogOutput logStream;

    /** Gets the log output stream object for this application */
    public LogOutput getLogPrinter () {
        return logStream;
    }

    /** Prints a message on the default log stream */
    final public void logOutput(String message) {
        logStream.logOutput(message);
    }

    /** Prints a message of type messageType on the log stream */
    final public void logOutput(String messageType, String message) {
        logStream.logOutput(messageType, message);
    }

    /** Prints a error message on the default log stream */
    final public void errorOutput(String message) {
        logStream.logErrorOutput(message);
    }

    /** Prints a error message of type messageType on the log stream */
    final public void errorOutput(String messageType, String message) {
        logStream.logErrorOutput(messageType, message);
    }

    /** Prints a debug message on the default log stream */
    final public void debugOutput(String message) {
        logStream.logDebugOutput(message);
    }

    /** Prints a debug message of type messageType on the log stream */
    final public void debugOutput(String messageType, String message) {
        logStream.logDebugOutput(messageType, message);
    }

    /** Prints a debug message on the default log stream when the debug level is set. */
    final public void debugOutput(String message, int debugLevel) {
        logStream.logDebugOutput(message, debugLevel);
    }

    /** Prints a debug message of type messageType on the log stream when the debug level is set. */
    final public void debugOutput(String messageType, String message, int debugLevel) {
        logStream.logDebugOutput(messageType, message, debugLevel);
    }

// options -----------------------------------------------------------------------------------------

    /** simulation options object */
    private SimOptions options;

    /** Gets the object holding the simulation options. */
    public SimOptions getOptions() {
        return options;
    }

    
// standalone --------------------------------------------------------------------------------------

	/** flag to specify if the application was started as an applet or as application */
    private boolean standalone= true;

    /** Tests if application is started standalone */
    public boolean isStandalone() {
        return standalone;
    }

    /** Sets if the application is started standalone or as applet */
    public void setStandalone(boolean a) {
        standalone = a;
    }


} // end SimlationApplication
