/**
 * Project mosaic
 *
 * SimMainFrame.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.variable.*;
import mosaic.util.img.*;
import java.beans.PropertyChangeEvent;

//=================================================================================================/

/**
* Base class for simulation frame windows.
* Has a set of visible simulation panels.
* Add all visible simulation panels to this main frame.
* Has the simulation control buttons and menus.
* <p>
* @see <{mosaic.sim.kernel.SimpleMainFrame}>
* @see <{TabbedMainFrame}>
*/

public class SimMainFrame extends JFrame implements ActionListener,
    ChangeListener, VariableChangeListener {

    /** for debug information */
    public static boolean[] DEBUG = new boolean[] {true};

    /** path separator character */
    final String separator = "\\";

// icons for the simulation control buttons  -------------------------------------------------------

    private ImageIcon playIcon;
    private ImageIcon stopIcon;
    private ImageIcon stepIcon;
    private ImageIcon resetIcon;
    private ImageIcon infoIcon;
    private ImageIcon redInfoIcon;
    private ImageIcon exitIcon;

// Toolbar and menu components ---------------------------------------------------------------------

    JToolBar toolBar;
    JButton btPlay;
    JButton btStop;
    JButton btStep;
    JButton btReset;
    JButton btOptions;
    JSlider slider;
    JLabel actSimTime;
    JLabel timeLabel;

    JMenuBar menuBar;
    protected JMenu menuView;
    JCheckBox cbOverview;
    JCheckBox cbDetail;
    JCheckBox cbTrend;
    JCheckBox cbAlarm;
    JButton btInfo;
    JButton btExit;

    /** Labels for the slider */
    private Hashtable slideLabel = new Hashtable();

// constructor -------------------------------------------------------------------------------------

    /** constructor to initialize the frame based on the application */
    public SimMainFrame(SimApplication appl) {

        setSimApplication (appl);
	    getSimApplication().setMainFrame(this);
        simulationOptionsDlg = new SimOptionsDlg (this, "SimulationOptions", true);
        simulationControlDlg = new SimControlDialog (this);
        simulationAboutDlg = new SimAboutDialog (this, "Info", false);

        // icons
        playIcon = Img.createSimImageIcon("play.gif");
        stopIcon = Img.createSimImageIcon("stop.gif");
        stepIcon = Img.createSimImageIcon("step.gif");
        resetIcon = Img.createSimImageIcon("reset.gif");
        infoIcon = Img.createSimImageIcon("info.gif");
        redInfoIcon = Img.createSimImageIcon("info_red.gif");
        exitIcon = Img.createSimImageIcon("exit.gif");

        // controls
        btPlay = new JButton(playIcon);
        btStop = new JButton(stopIcon);
        btStep = new JButton(stepIcon);
        btReset = new JButton(resetIcon);
        btOptions = new JButton("Options");
        slider = new JSlider(0,100,50);
        actSimTime = new JLabel("0.0");
        timeLabel = new JLabel("Sim.Time:");
        cbOverview = new JCheckBox("Overview");
        cbDetail = new JCheckBox("Detail");
        cbTrend = new JCheckBox("Trend");
        cbAlarm = new JCheckBox("Alarm");
        btInfo = new JButton(infoIcon);
        btExit = new JButton(exitIcon); 

        // listen to time variable
	    getSimApplication().getTimeVariable().addVariableChangeListener(this);

	    // title
        setTitle("variable Frame");

	    // Menu in the toolbar
        menuBar = new JMenuBar();
        menuView = new JMenu("View");
	    menuView.setPreferredSize(new Dimension(80,menuView.getSize().height));
	    menuBar.add(menuView);

	    // Buttons
	    btPlay.setActionCommand("play");
	    btStep.setActionCommand("step");
	    btStop.setActionCommand("stop");
	    btReset.setActionCommand("reset");
        btOptions.setActionCommand("options");
	    btInfo.setActionCommand("info");
	    btExit.setActionCommand("exit");
	    cbOverview.setActionCommand("overview");
	    cbDetail.setActionCommand("detail");
	    cbTrend.setActionCommand("trend");
	    cbAlarm.setActionCommand("alarm");

	    // Labels for Slider in toolbar
	    slideLabel.put(new Integer(0),new JLabel("slower"));
	    slideLabel.put(new Integer(50),new JLabel("normal"));
	    slideLabel.put(new Integer(100),new JLabel("faster"));

	    // Slider
	    slider.setMajorTickSpacing(10);
	    slider.setLabelTable(slideLabel);
	    slider.setPreferredSize(new Dimension(200,slider.getSize().height));
	    slider.setPaintTicks(true);
	    slider.setPaintLabels(true);
	    getSimApplication().getOptions().setAnimationRate(1000);
	    slider.addChangeListener(this);

	    // Setting up alignment
	    btPlay.setAlignmentY(TOP_ALIGNMENT);
	    btStep.setAlignmentY(TOP_ALIGNMENT);
	    btStop.setAlignmentY(TOP_ALIGNMENT);
	    btReset.setAlignmentY(TOP_ALIGNMENT);
	    slider.setAlignmentY(TOP_ALIGNMENT);
	    timeLabel.setAlignmentY(TOP_ALIGNMENT);
	    actSimTime.setAlignmentY(TOP_ALIGNMENT);
	    menuBar.setAlignmentY(TOP_ALIGNMENT);
        btOptions.setAlignmentY(TOP_ALIGNMENT);
	    btInfo.setAlignmentY(TOP_ALIGNMENT);
	    btExit.setAlignmentY(TOP_ALIGNMENT);

        // ToolBar
        toolBar = new JToolBar();
        toolBar.add(btPlay);
        toolBar.addSeparator();
        toolBar.add(btStep);
        toolBar.addSeparator();
        toolBar.add(btStop);
        toolBar.addSeparator();
        toolBar.add(btReset);
        toolBar.addSeparator();
        toolBar.add(slider);
        toolBar.addSeparator();
        toolBar.add(timeLabel);
        toolBar.addSeparator();
        toolBar.add(actSimTime);
        toolBar.add(Box.createHorizontalGlue());
        toolBar.add(menuBar);
        toolBar.addSeparator();
        toolBar.add(btOptions);
        toolBar.add(btInfo);
        toolBar.addSeparator();
        toolBar.add(btExit);

        // layout
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(toolBar, BorderLayout.NORTH);

        // actionlistener for buttons in toolbar
        btPlay.addActionListener(this);
        btStep.addActionListener(this);
        btStop.addActionListener(this);
        btReset.addActionListener(this);
        btOptions.addActionListener(this);
        btInfo.addActionListener(this);
        btExit.addActionListener(this);

        // tooltips
        setToolTips();

        // window closing listener
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                this_windowClosing(e); }
                });
                
        //jbInit
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace(getSimApplication().getLogPrinter().getLogStream("error"));
        }
    }

    /** Initialization of frame. */
    private void jbInit() throws Exception {
    }

// model panels ------------------------------------------------------------------------------------

    Vector modelPanels = new Vector ();

    /** Add a panel to the vector of simulation panels */
    public void addModelPanel (ModelPanel panel) {
        modelPanels.addElement(panel);
        getSimApplication().addModelComponent(panel);
    }

    /** Remove a panel to the vector of simulation panels */
    public void removeModelPanel (ModelPanel panel) {
        modelPanels.removeElement(panel);
        getSimApplication().removeModelComponent(panel);
    }

    /** Gets the array of SimulationPanels */
    public ModelPanel[] getModelPanels () {
        return ((ModelPanel[])modelPanels.toArray(new ModelPanel[0]));
    }

// dialog windows  ---------------------------------------------------------------------------------

    //simulation control dialog
    SimControlDialog simulationControlDlg;

    //simulation options dialog
    //SimulationOptionsDialog simulationOptionsDlg = new SimulationOptionsDialog (this, "SimulationOptions", false);
    //Hilpold 22.03.00
    SimOptionsDlg simulationOptionsDlg;

    //simulation options dialog
    SimAboutDialog simulationAboutDlg;

    public void closeAllDialogs() {
        this.simulationAboutDlg.dispose();
        this.simulationControlDlg.dispose();
        this.simulationOptionsDlg.dispose();
    }

    /** setting up tooltips for toolbar */
    private void setToolTips() {

        btPlay.setToolTipText("Play");
        btStop.setToolTipText("Stop");
        btStep.setToolTipText("Step");
        btReset.setToolTipText("Reset");
        btInfo.setToolTipText("Info Mode");
        btExit.setToolTipText("Exit");
    }


// slider ------------------------------------------------------------------------------------------

    double mul = 50;

    /** Reacts to statechanged of jslider */
    public void stateChanged(ChangeEvent e) {
        if (e.getSource() == slider) {
            double rate;
            getSimApplication().debugOutput("SliderValue: "+slider.getValue());
            if (slider.getValue() < 47) {
                rate = slider.getValue() * ((1 - 1 / mul) / 50)  + 1/mul;
            } else if (slider.getValue() > 53) {
                rate = slider.getValue() * ((mul  - 1) / 50)  + 2 - mul;
            } else {
                // true realtime
                rate = 1;
            }
            getSimApplication().debugOutput("New animation rate: "+rate);
        getSimApplication().getOptions().setAnimationRate((long)(rate * 1000));
        }
    }

// action listener ---------------------------------------------------------------------------------

    /** Reacts to an action performed event of the listened controls. */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("play")) {
            doStart();
        } else if (e.getActionCommand().equals("step")) {
            doStep();
        } else if (e.getActionCommand().equals("stop")) {
            doStop();
        } else if (e.getActionCommand().equals("reset")) {
            doReset();
        } else if (e.getActionCommand().equals("options")) {
            doOpenOptionsDlg();
        } else if (e.getActionCommand().equals("info")) {
            doToggleInfoMode();
        } else if (e.getActionCommand().equals("exit")) {
            doExit();
        } else {
            getSimApplication().errorOutput("SimMainFrame: could not eval actionCommand " + e.toString());
        }
    }

// Interactive operations --------------------------------------------------------------------------

    /** Exits the application */
    public void doExit () {
        getSimApplication().getSimulationScheduler().stop();
        dispose();
        if (getSimApplication().isStandalone())
            System.exit(0);
    }

    /** Resets the simulation */
    public void doReset () {
        getSimApplication().reset();
    }

    /** Starts the simulation */
    public void doStart () {
        if (!getSimApplication().getSimulator().isRunning()) {
            getSimApplication().getSimulationScheduler().start();
        }
        btPlay.setEnabled(false);
        btStop.setEnabled(true);
    }


    /** Stops the simulation */
    public void doStop () {
        if (getSimApplication().getSimulator().isRunning()) {
            getSimApplication().getSimulationScheduler().stop();
        }
        btStop.setEnabled(false);
        btPlay.setEnabled(true);
    }

    /** Does one event step */
    public void doStep () {
        if (!getSimApplication().getSimulator().isRunning()) {
            getSimApplication().getSimulationScheduler().step();
        }
    }

    /** Generates a debug information about the simulation */
    public void doDebug () {
        getSimApplication().debugOutput("simulation", " event list: " +
                getSimApplication().getSimulator().getEventListDebugString());
    }

    /** Opens the option dialog */
    public void doOpenOptionsDlg () {
        getSimApplication().errorOutput("SimMainFrame: doOpenOptions called");
        if (!getSimApplication().getSimulationScheduler().isRunning()) {
            simulationOptionsDlg.update();
            simulationOptionsDlg.setVisible(true);
        } else {
            getSimApplication().logOutput("Stop simulation before opening Options Dlg");
        }
    }

    /** Opens the simulation control dialog. */
    public void doOpenControlDlg () {
        simulationControlDlg.setVisible(true);
        simulationControlDlg.setLocation(100,100);

    }

    /** Opens the about dialog. */
    public void doOpenAboutDlg () {
        simulationAboutDlg.show();
    }

    // Property and menuaccess to switch actual infoMode
    boolean infoMode = false;

    public boolean isInfoMode() {
        return infoMode;
    }

    /** Opens the about dialog. */
    public void doToggleInfoMode () {
        if (infoMode) {
            infoMode = false;

    	    btInfo.setIcon(infoIcon);
            infoCursor = new Cursor(Cursor.DEFAULT_CURSOR);
            this.firePropertyChange("infoMode",
                                    new Boolean(true),
                                    new Boolean(false));
        } else {
                infoMode = true;
                btInfo.setIcon(redInfoIcon);
                infoCursor = new Cursor(Cursor.HAND_CURSOR);
                this.firePropertyChange("infoMode",
                                        new Boolean(false),
                                        new Boolean(true));
        }
    }

    /** Cursor for the info mode */
    static Cursor infoCursor = new Cursor(Cursor.DEFAULT_CURSOR);
    /** Gets the cursor for the info mode */
    public static Cursor getInfoCursor() {
        return infoCursor;
    }


// process the window message closing --------------------------------------------------------------

    /** Processed the WINDOW_CLOSING by exiting the application. */
    protected void processWindowEvent(WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            doExit();
        }
        super.processWindowEvent(e);
    }

    void this_windowClosing(WindowEvent e) {
        if (getSimApplication().isStandalone())
	    System.exit(0);
    }

// SimApplication property -------------------------------------------------------------------------

    protected SimApplication application;

    /** Gets the simulation application object */
    public SimApplication getSimApplication() {
        return application;
    }

    /** Sets the simulation application object */
    public  void setSimApplication(SimApplication appl) {
        application = appl;
    }

// Variable change listener of time variable -------------------------------------------------------

    /** Reacts to variableChanges from time variable  */
    public void variableChange(VariableChangeEvent evt) {
        String timeString;
        if (getSimApplication().getTime() >= Double.MAX_VALUE) {
            timeString = "INF";
        } else {
            timeString = "" + getSimApplication().getTime();
            if (timeString.length() > 4) {
                timeString = timeString.substring(0, 3);
            }
        }
    	actSimTime.setText(" "  + timeString);
    }

    /** Reacts to a variable changed event. */
    public void propertyChange (PropertyChangeEvent e) {
    }

    /** Adds a listener variable. */
    public void addListenerVariable (Variable v){
    }

    /** Removes a listener variable. */
    public void removeListenerVariable (Variable v) {
    }

    /** variable to hold the priority for handling this listener in handling variable change events. */
    protected int priority = 1;

    /**
     * Gets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final int getPriority() {
        return priority;
    }

    /**
     * Sets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final void setPriority(int priority) {
        this.priority = priority;
    }


} // end SimMainFrame ==============================================================================

