package mosaic.sim.kernel.continuous;

import java.awt.*;
import java.awt.geom.*;
import java.awt.Color;
import java.util.*;
import javax.swing.JLabel;
import java.beans.Beans;

import mosaic.sim.kernel.*;
import mosaic.sim.kernel.variable.*;

/**
* This is an abstract implementation of the ContEquation interface.
* To create a concrete class, override the equation method and set
* the used domain and target variable in the constructor.
*/

public abstract class AbstractContEquation extends AbstractModelComponent implements ContEquation{
   JLabel la;

   protected DoubleVariable[] targets={};
   protected DoubleVariable[] domains={};

   protected boolean continuousMode=false;

   protected int countContinuousVariables=0;

   public AbstractContEquation() {
      this("equation");
      } // AbstractContEquation;

   public AbstractContEquation(String name) {
      super(name);
      } // AbstractContEquation

   public AbstractContEquation(DoubleVariable[] domains,DoubleVariable[] targets){
      this ("equation", domains, targets);
   } // AbstractContEquation;

   public AbstractContEquation(String name,DoubleVariable[] domains,DoubleVariable[] targets){
      super(name);
      try{
        setDomains(domains);
        setTargets(targets);
      }catch(VariableException e){System.out.println(e.getMessage());};
   } // AbstractContEquation

    public boolean isContinuousMode(){
        return continuousMode;
    }

    protected void getContinuous(){
        if (!continuousMode){
            continuousMode=true;
            fireModelComponentGotContinuous();
            if (enabled){
                for (int i=0;i<targets.length;i++){
                    if (targets[i]!=null && targets[i] instanceof DoubleVariable){
                        ((DoubleVariable)targets[i]).inCouplerGotContinuous();
                    }
                }
            }
        }
    }

    protected void getDiscrete(){
        if (continuousMode){
            continuousMode=false;
            fireModelComponentGotDiscrete();
            if (enabled){
                for (int i=0;i<targets.length;i++){
                    if (targets[i]!=null && targets[i] instanceof DoubleVariable){
                        ((DoubleVariable)targets[i]).inCouplerGotDiscrete();
                    }
                }
            }
        }
    }

    public void domainGotDiscrete(){
        countContinuousVariables--;
        if (countContinuousVariables==0) getDiscrete();
    }

    public void domainGotContinuous(){
        countContinuousVariables++;
        getContinuous();
    }

   /** Tells the target variables that they are connected to this equation. */
   protected final void connectTargetVars(DoubleVariable[] targets) throws VariableException{
     for (int i=0;i<targets.length;i++) targets[i].setInCoupler(this);
   }

   /** Removes the connection from the target variables. */
   protected final void disconnectTargetVars(DoubleVariable[] targets){
     try{
       for (int i=0;i<targets.length;i++) targets[i].setInCoupler(null);
     }
     catch(VariableException e){} // will not happen if Coupler==null;
   }

   /** Tells the domain variables that they are connected to this equation. */
   protected final void connectDomainVars(DoubleVariable[] domains) throws VariableException{
     for (int i=0;i<domains.length;i++) domains[i].addOutCoupler(this);
   }

   /** Removes the connection from the domain variables. */
   protected final void disconnectDomainVars(DoubleVariable[] domains){
     for (int i=0;i<domains.length;i++) domains[i].removeOutCoupler(this);
   }

   /** Returns the target variables. */
   public Variable[] getTarget(){ return targets;}

   /** Returns the domain variables. */
   public Variable[] getDomain(){ return domains;}

   /**
   * Set the domain variables
   */
   protected final void setDomains (DoubleVariable[] domains) throws VariableException {
     if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
       getSimApplication().getIntegrator().stateEventOccurred = true;
       return;
     }
     if (this.domains==domains) return; // nothing changed
     if (enabled && this.domains!=null) disconnectDomainVars(this.domains);
     this.domains=domains; // set new domain
     if (enabled && domains!=null){
        connectDomainVars(domains);
        discreteUpdate();
     }
     checkCont();
     fireModelComponentChanged(); // inform listeners
   }

   /**
   * Set the target variable
   */
   protected final void setTargets (DoubleVariable[] targets) throws VariableException{
     if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
       getSimApplication().getIntegrator().stateEventOccurred = true;
       return;
     }
     if (this.targets==targets) return; // nothing changed
     if (enabled && this.targets!=null) disconnectTargetVars(this.targets);
     this.targets=targets; // set new target
     if (enabled && targets!=null){
        connectTargetVars(targets);
        discreteUpdate();
     }
     fireModelComponentChanged(); // inform listeners
   }

   /** enables the equation. */
   public void setEnabled(boolean enabled){
     if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
       getSimApplication().getIntegrator().stateEventOccurred = true;
       return;
     }
     super.setEnabled(enabled); // supercall to BasicModel
     if (enabled){
       try{
         connectDomainVars(domains);
         connectTargetVars(targets);
         checkCont();
         discreteUpdate();
       } // try to connect
       catch(VariableException e){System.out.println(e.toString());}
     }
     else{
       disconnectDomainVars(domains);
       disconnectTargetVars(targets);
     }
   }

    protected final void checkCont(){
        countContinuousVariables=0;
        for (int i=0;i<domains.length;i++){
           if (domains[i] instanceof DoubleVariable && ((DoubleVariable)domains[i]).isContinuous()){
             countContinuousVariables++;
           }
        }
        if (countContinuousVariables==0){
           getDiscrete();
        }else{
           getContinuous();
        }
    }

    protected final void discreteUpdate(){
        if (continuousMode||!enabled)return;
        try{
            equation();
        }catch(Exception e){}//if it fails, than do nothing!
    }

    public final void definedByChange(Variable d){
        discreteUpdate();
    }

    public final void propagateValueChange(Variable d){
        discreteUpdate();
    }

   public final void domainTypeChange(Variable d){
//     if (!enabled) return;
     fireModelComponentChanged(); // inform listeners
   }

   public final void targetTypeChange(Variable d){
//     if (!enabled) return;
     fireModelComponentChanged(); // inform listeners
   }

// connect of variables ----------------------------------------------------------------------------

    /** Connect one variable to the other by creating and adding a new BasicConnection */
    public void connect(Variable from, Variable to) throws VariableException {
        if (! (Beans.isInstanceOf(from, ((Object)to).getClass()))) {
            // type icompatibilites of variables - from must be a subtype of to
            throw new VariableException(from);
        }
        BasicConnection connection = new BasicConnection(from.getLabel() + "-" + to.getLabel());
        connection.setDomain(from);
        connection.setTarget(to);
        this.add(connection);
    }

    /** Unconnect the variable, ie., delete the coupling for this variable. */
    public void unconnect(Variable to) throws VariableException {
        try {
            Coupler toCoupler = to.getInCoupler();
            if (toCoupler != null && toCoupler instanceof Connection) {
                Connection toConnection = (Connection) toCoupler;
                toConnection.setDomain(null);
                toConnection.setTarget(null);
                this.remove((AbstractModelComponent)toConnection);
            }
        } catch (ClassCastException e) {
            printErrorOutput("Not possible to resolve the coupling for the variable: " + to.getLabel());
        }
    }

// equation ----------------------------------------------------------------------------------------

   /**
   *  Method to implement the continuous equation (dummy here).
   */
   public void equation() throws ContSimulationException {
   }

// -------------------------------------------------------------------------------------------------

   /** Disconnect befor deletion. */
   protected void finalize(){
     try{
       super.finalize();
     }catch(Throwable t){}
     disable();
   }
  protected void initPaint(){
    la=new JLabel();
    la.setFont(new java.awt.Font("Dialog", 1, 10));
    la.setForeground(Color.black);
    la.setOpaque(false);
  }

    /**
    * Paints the connection.
    * Paints an arrow and labels it with name of domain and target.
    * @param g      the graphics context of the component
    */
  public void paintModelComponent(Graphics g) {
    Graphics2D g2=(Graphics2D) g;
    int h = getSize().height-1;
    int w = getSize().width-1;
    int minWH=Math.min(h,w/2);
    int dist=Math.max(h/10,2);
    initPaint();

    Color lineC;
    Color fillC;
/*    if (enabled) g.setColor(Color.black);
    else g.setColor(Color.gray);*/
    if (enabled){
      lineC=Color.black;
      fillC=Color.yellow;
    }
    else{
      lineC=Color.gray;
      fillC=Color.lightGray;
    }
    g2.setPaint(fillC);

//    Rectangle r=new Rectangle(w/4,0,w/2,h);
    Ellipse2D e=new Ellipse2D.Double(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    g2.fill(e);


    g2.setPaint(lineC);

    GeneralPath p=new GeneralPath(GeneralPath.WIND_EVEN_ODD);

    g2.draw(e);

    p.moveTo(0,h/2+dist);
    p.lineTo(0,h/2-dist);
    p.lineTo(w/2-minWH/2-dist*2,h/2-dist);
    p.lineTo(w/2-minWH/2-dist*2,h/2-dist*2);
    p.lineTo(w/2-minWH/2,h/2);
    p.lineTo(w/2-minWH/2-dist*2,h/2+dist*2);
    p.lineTo(w/2-minWH/2-dist*2,h/2+dist);
    p.closePath();
    g2.draw(p);

//    g2.draw(new Line2D.Double(0,h/2+dist,w/2-minWH/2,h/2+dist));
//    g2.draw(new Line2D.Double(0,h/2-dist,w/2-minWH/2,h/2-dist));

//    g2.draw(new Line2D.Double(0,h/2,w/2-minWH/2,h/2));
//    g2.draw(new Line2D.Double(w/2+minWH/2,h/2,w,h/2));
    AffineTransform trans=new AffineTransform();
    trans.translate((minWH+w)/2,0);
    p.transform(trans);
    g2.draw(p);

//    la.setBounds((x1+x2)/2+5,(y1+y2)/2-10,(int)la.getPreferredSize().getWidth(),(int)la.getPreferredSize().getHeight());
    la.setHorizontalAlignment(la.CENTER);
    la.setVerticalAlignment(la.CENTER);

    la.setBounds(e.getBounds());
    Graphics2D g3=(Graphics2D)g2.create(w/2-minWH/2,h/2-minWH/2,minWH,minWH);
    la.setText("EQ");
    la.paint(g3);

    String input="";
    String output="";
    for (int i=0;i<domains.length;i++){
      input=input+domains[i].getLabel();
      if (i!=(domains.length-1)) input=input+",";
    }
    for (int i=0;i<targets.length;i++){
      output=output+targets[i].getLabel();
      if (i!=(targets.length-1)) output=output+",";
    }

    if (domains.length>0){
      la.setBounds(0,0,w/2-minWH/2,h/2-dist);
      la.setText(input);
      la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.setHorizontalAlignment(la.LEFT);
      g3=(Graphics2D)g2.create(0,0,w/2-minWH/2,h/2-dist);
      la.paint(g3);
    }

    if (targets.length>0){
      la.setBounds(w/2+minWH/2,0,w/2-minWH/2,h/2-dist);
      la.setText(output);
      la.setHorizontalAlignment(la.RIGHT);
      g3=(Graphics2D)g2.create(w/2+minWH/2,0,w/2-minWH/2,h/2-dist);
      if (domains.length<=0)
        la.setFont(new java.awt.Font("Dialog", 0, 10));
      la.setVerticalAlignment(la.BOTTOM);
      la.paint(g3);
    }

  }

}
// end of BasicConnection.java
