
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// interface VariableStatistic    ----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.statistics;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.control.*;
import mosaic.sim.kernel.variable.*;

/**
* Class for computing statistics for values of numerical Variables.
* Is a VariableChangeListener. Listens to the values over time of the Variable,
* and computes a time dependent statistic.
* Uses a TimeDependentStatistic object to do the computation.
* <p>
* <b>Usage: </b>
* <ul>
* <li> Use this component to compute a time weighted statistics over a variable value.
* <li> Works for numeric and also boolean (!) values. 
* <li> Has the capability to open a report window when clicked on the bean and give out the current statistics.
* </ul>
*
* <b>Bound properties:</b>
* <dl>
* <dt><code> min      </code></dt> <dd> the minimum value received </dd>
* <dt><code> max      </code></dt> <dd> the maximum value received </dd>
* <dt><code> mean     </code></dt> <dd> the mean of the values received </dd>
* <dt><code> stdDev   </code></dt> <dd> the std deviaion of  values received </dd>
* </dl>
*
* <p>
* <b>Costumization: </b>
* <ul>
* <li> set the label of the statistics bean by <code>setLabel</code>
* <li> set the descriptioin of the statistics bean used as header in the output by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*
*
* @see   TimeDependentStatistic
*
* @author Herbert Praehofer
* @version %I% %G%
*/

public class VariableStatistic extends BasicVariableObserver
{
    
  /** Default constructor VariableStatistic */
  public VariableStatistic() {
      this(" ");
  }

  /** Default constructor VariableStatistic */
  public VariableStatistic (String description) {
      setLabel (description);
      add (statistic);
      addMouseListener(new ReportWindowOpener());
  }


// Statistic object ------------------------------------------------------------------------------------------

  TimeDependentStatistic statistic = new TimeDependentStatistic ();


// variableChange --------------------------------------------------------------------------------------------

  /**
  * Reacts to a variableChange event by setting the value and repainting the view.
  */
  public void variableChange(VariableChangeEvent e) {
    Object value = e.getValue();
    if (value instanceof Number) {
        statistic.newObservation(value);
    } else if (value instanceof Collection) {
        statistic.newObservation(new Integer(((Collection)value).size()));
    } else {
        printErrorOutput("tried to send not a number or collection object to an VariableStatistic " + getLabel());
    }
  }

// -----------------------------------------------------------------------------------------------------------
// statistic values ------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

// min  ------------------------------------------------------------------------------------------------------

  /** Gets the minimum value */
  public double getMin () {
        return statistic.getMin();
  }

// max  ------------------------------------------------------------------------------------------------------

  /** Gets the maximum value */
  public double getMax () {
        return statistic.getMax();
  }

// mean   ----------------------------------------------------------------------------------------------------

  /** Gets the mean value */
  public double getMean () {
        return statistic.getMean();
  }


// standard deviation    -------------------------------------------------------------------------------------

  /** Gets the stdDev value */
  public double getStdDev () {
        return statistic.getStandardDeviation();
  }

     
// reset -----------------------------------------------------------------------------------------------------

  /** reset the output bean */
  public void reset () {
        super.reset();
        statistic.reset();
  }

// size ------------------------------------------------------------------------------------------------------

  /** Gets minimum size for the LayoutManager */
  public Dimension getMinimumSize() {
      return new Dimension(5,5);
  }

  /** Gets preferred size for the LayoutManager */
  public Dimension getPreferredSize() {
      return new Dimension(10,10);
  }

// paint -----------------------------------------------------------------------------------------------------
 
  /**
  * Paints the view;
  * by calling <code>super.paint</code> which paints the background and
  * calling polymorphic method <code>paintValue</code> to delegate painting of the values to specialized views. 
  *
  * @param g the graphics context of the view
  */
  public void paintComponent(Graphics g) {
    int width = getSize().width;
    int height = getSize().height;
    g.setColor(getBackground());
    g.fill3DRect(0, 0, width, height, true);
    g.setColor(getForeground());
    g.setFont(getFont());
    FontMetrics fm = g.getFontMetrics();
    g.drawString("?" + getLabel(), (width - fm.stringWidth(getLabel())) / 2,
    height - fm.getMaxDescent() -2);
//    super.paint(g);
  }


// -----------------------------------------------------------------------------------------------------------
// report output in dialog
// -----------------------------------------------------------------------------------------------------------

  protected StatisticsReportWindow reportW;
  private String description = "";

  /**
  * Inner class to react to mouse events by opening a report window and printing a report about the statistics
  */
  protected Frame getParentFrame() {
        // look for parent frame window to create Dialog
        Container p = getParent();
        while (!((p == null) || (p instanceof Frame))) {
            p = p.getParent();
        }
        if (p == null)
            return null;
        else
            return (Frame)p;
    }
    
  class ReportWindowOpener extends MouseAdapter {
        public void mouseClicked (MouseEvent e) {
            if (reportW == null) {
                Frame f = getParentFrame();
                if (f != null & f instanceof Frame) {
                    reportW = new StatisticsReportWindow (f, "Statistics " + getLabel());
                    reportW.pack();
                    reportW.setModal(false);
                }
                else {
                    System.out.println(">> tried to open report window without parent frame for statistics bean " + getLabel());
                }
            }
            if (reportW != null) {
                reportW.show();
                reportW.toFront();
                report(reportW);
            }
        }
  }

  /** Reports the current statistics information in a report window */
  public void report (StatisticsReportWindow reportW) {
        TextArea reportArea = reportW.getTextArea();
        reportArea.setText("");
        reportArea.append (getDescription() + "\n");
        reportArea.append ("_________________________________________" + "\n\n");
        reportArea.append ("  from start time " + statistic.getStartTime() + " to current time " + getTime() + "\n\n");
        reportArea.append ("  Minimum:      " + statistic.getMin() + "\n");
        reportArea.append ("  Maximum:      " + statistic.getMax() + "\n");
        reportArea.append ("  Means:        " + statistic.getMean() + "\n");
        //reportArea.append ("  StdDeviation: " + statistic.getStandardDeviation() + "\n");
  }



  /** Reports the current statistics information in a report window */
  public void report (java.io.DataOutputStream out) throws java.io.IOException {
      out.writeBytes(getDescription() + "\n");
      out.writeBytes("_________________________________________" + "\n\n");
      statistic.report(out);

  }

  public void setDescription(String newDescription) {
    description = newDescription;
  }

  public String getDescription() {
      return description;
  }


}

