/**
 * Project mosaic
 *
 * AbstractVariable.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel.variable;

import java.util.*;
import mosaic.sim.kernel.control.Simulator;
import concurrent.CopyOnWriteArrayList;
import mosaic.sim.kernel.AbstractModelComponent;
import mosaic.sim.kernel.simevent.*;


/**
* Class AbstractVariable is an abstract root class for all different types of model variables.
* Variables are used to implement data fields (state, input, output, constants) in models.
* Implements an standard mechanism to signal variableChangeEvents, that is,
* they signal the variableChangeEvent to observers in synchroneous and to models in asynchroneous way.
* <P>
* Implements the ModelComponent, Variable, and InputEventListener interfaces.
* Has unit, type, unit, ...
* Has a method <code>getValueObject</code> to get the Object wrapper for the value of the Variable.
* Concrete subclasses of Variable, like IntVariable, have values of different types,
* and methods <code>getValue</code> and <code>setValue</code> to read and write them.
* Variables can be connected.
*/
abstract public class AbstractVariable
         extends AbstractModelComponent
         implements Variable, InputEventListener  {

// constants and fields ----------------------------------------------------------------------------

    /** Variable definedBy defines the final defining Variable in connected variables. */
    protected Variable definedBy=this;

    /** All connections using this variable as source */
    protected CopyOnWriteArrayList outCouplers=new CopyOnWriteArrayList();

    /** All connections using this variable as target */
    protected mosaic.sim.kernel.variable.Coupler inCoupler;

    /** Unit of the variable. */
    protected String unit;

    /** Quantity of the variable. */
    protected String quantity;

    /** Unit for output. */
    protected String displayUnit;

    /** Type of the variable (STATE, INPUT, OUTPUT, CONSTANT, VARIABLE) */
    protected int type;


// Constructors ------------------------------------------------------------------------------------

   /** Default constructor */
   public AbstractVariable() {
        this("V", VARIABLE, "", "", "");
     }

   /** Constructor initializing _parentModel and providing a name */
   public AbstractVariable(String name) {
        this(name, VARIABLE, "", "", "");
       }

   /** Constructor initializing _parentModel, providing a name, and defining the type */
   public AbstractVariable(String name, int type) {
        this(name, type, "", "", "");
       }

   /** Constructor initializing _parentModel, providing a name, description, unit,
   * quantity displayUnit, and defining the type */
   public AbstractVariable(String name, int type, String unit, String quantity, String displayUnit) {

      super(name);

      setUnit(unit);
      setQuantity(quantity);
      setDisplayUnit(displayUnit);

      definedBy  = this;

      if ((type != STATE)&&(type != INPUT)&&(type != OUTPUT)&&(type != CONSTANT)&&(type != VARIABLE))
         this.type = VARIABLE;
      else
         this.type = type;
      changedTime = NEGINFINITY;
   }


// variable type -----------------------------------------------------------------------------------

   /** Tests if the variable is a STATE. */
   public final boolean isTypeState()
      { return (type == STATE); }

   /** Tests if the variable is a INPUT. */
   public final boolean isTypeInput()
      { return (type == INPUT); }

   /** Tests if the variable is a OUTPUT. */
   public final boolean isTypeOutput()
      { return (type == OUTPUT); }

   /** Tests if the variable is a CONSTANT. */
   public final boolean isTypeConstant()
      { return (type == CONSTANT); }

   /** Tests if the variable is a VARIABLE. */
   public final boolean isTypeVariable()
      { return (type == VARIABLE); }

   /** Sets the type of the variable to STATE. */
   public final void setTypeState()
      { setType(STATE); }

   /** Sets the type of the variable to INPUT. */
   public final void setTypeInput()
      { setType(INPUT); }

   /** Sets the type of the variable to OUTPUT. */
   public final void setTypeOutput()
      { setType(OUTPUT); }

   /** Sets the type of the variable to CONSTANT. */
   public final void setTypeConstant()
      { setType(CONSTANT); }

   /** Sets the type of the variable to VARIABLE. */
   public final void setTypeVariable()
      { setType(VARIABLE); }

    protected final void setType(int type){
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        if (this.type==type) return;
        this.type=type;
        fireModelComponentChanged();
    }


// unit, quanity  ----------------------------------------------------------------------------------

   /** Gets the unit of the variable. */
   public final String getUnit()
      { return unit; }

   /** Gets the quantity of the variable. */
   public final String getQuantity()
      { return quantity; }

   /** Gets the display unit of the variable. */
   public final String getDisplayUnit()
      { return displayUnit; }

   /** Sets the unit of the variable. */
   public final void setUnit(String unit)
      { this.unit = unit;}

   /** Sets the quantity of the variable. */
   public final void setQuantity(String quantity)
      { this.quantity = quantity; }

   /** Sets the display unit of the variable. */
   public final void setDisplayUnit(String displayUnit)
      { this.displayUnit = displayUnit; }


// variableChangeEvent -----------------------------------------------------------------------------

    /** Vector to hold the VariableChangeListeners */
    protected CopyOnWriteArrayList variableChangeListeners = new CopyOnWriteArrayList();

    /** Add a VariableChangeListener either to the variable */
    public void addVariableChangeListener (VariableChangeListener listener) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        variableChangeListeners.add (listener);
        getSimApplication().getSimulator().registerChangeEvent(new VariableChangeRegistry(this, listener));
        listener.addListenerVariable(this);
    }

    /** Remove a VariableChangeListener */
    public void removeVariableChangeListener (VariableChangeListener listener) {
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        variableChangeListeners.remove (listener);
        listener.removeListenerVariable(this);
    }


// signal variable changes -------------------------------------------------------------------------

    /*
    * Signals a variable change event of this variable.
    * Sets the time of the change.
    * Registers the change event for the model listeners (asynchroneous event).
    * Calls all listeners of type observer immediatly (synchroneous event).
    * Propagates the change to all connected variables via the connections.
    */
    public final void signalVariableChange () {

        if (getSimApplication() == null || isDisabled()) {
            return;
        }

        // set the time when the variable has changed
        setChangedTime();

        // register a variable change at the simulator
        if (! variableChangeListeners.isEmpty()){
            Simulator simulator = getSimApplication().getSimulator();
            VariableChangeListener listener;
            Iterator listenerElems = variableChangeListeners.iterator();
            while (listenerElems.hasNext()) {
                listener = ((VariableChangeListener) listenerElems.next());
                simulator.registerChangeEvent(new VariableChangeRegistry(this, listener));
            }
        }

        // propagate the variable change to the connected variables.
        if (outCouplers.size()>0){
            Iterator outgoingCouplers=outCouplers.iterator();
            while(outgoingCouplers.hasNext()){
                Coupler c=(Coupler)outgoingCouplers.next();
                c.propagateValueChange(this);
            }
        }
    }

    /**
    * Signals a variable change event of this variable directly (without a registration at the simulator).
    * Sets the time of the change.
    * Class the method variableChange of the listenes.
    * Does not propagates the change to the connected variables via the connections because they are
    * continuous also.
    * @see signalVariableChange
    */
    public final void signalVariableChangeDirectly () {
        if (getSimApplication() == null) {
            return;
        }
        // set the time when the variable has changed
        setChangedTime();

        // signal the events in directly
        if (! variableChangeListeners.isEmpty()){
            Simulator simulator = getSimApplication().getSimulator();
            VariableChangeListener listener;
            Iterator listenerElems = variableChangeListeners.iterator();
            while (listenerElems.hasNext()) {
                listener = ((VariableChangeListener) listenerElems.next());
                listener.variableChange(new VariableChangeEvent(this, getValueObject(), getChangedTime()));
            }
        }

        // do not propagate the variable change to the connected variables because they are continuous anyway.
    }


// changed time ------------------------------------------------------------------------------------

    /** time the last change occurred in this variable   */
    protected double changedTime;

    /** Gets the time of the last change in the variable */
    public double getChangedTime() {
        return changedTime;
    }

    /** Sets the time of the last change in the variable */
    public void setChangedTime () {
        changedTime = getTime();
    }

    /** Sets the time of the last change in the variable */
    public void setChangedTime (double time) {
        changedTime = time;
    }

    /** Clears the time of the last change in the variable */
    public void clearChangedTime () {
        changedTime = NEGINFINITY;
    }

    /** Get all VariableChangeListener. */
    public Collection getVariableChangeListener(){
        return variableChangeListeners;
    }

// connect      ------------------------------------------------------------------------------------

    /** Adds a coupler that listens to this variable. */
    public final void addOutCoupler(mosaic.sim.kernel.variable.Coupler c){
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        if (!outCouplers.contains(c)){
            outCouplers.add(c);
        }
    }

    /** Removes a coupler that listens to this variable. */
    public final void removeOutCoupler(mosaic.sim.kernel.variable.Coupler c){
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
        return;
        }
        outCouplers.remove(c);
    }

    /** Adds a coupler that sets this variable. To remove, set to null*/
    public void setInCoupler(mosaic.sim.kernel.variable.Coupler c)throws VariableException{
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
        return;
        }
        if (inCoupler==c) return;
        if (c!=null && inCoupler!=null) throw new VariableException("Variable already defined ", this);
        inCoupler=c;
    }

    /** Returns the ingoing coupler. */
    public final mosaic.sim.kernel.variable.Coupler getInCoupler(){
        return inCoupler;
    }

    /** Returns all listening couplers. */
    public final List getOutCouplers(){
        return outCouplers;
    }

    /** Don't use this method!
 *     Use a new Connection object */
    public final void connect(Variable variable)throws VariableException{
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        new BasicConnection(this,variable);
    }

    /** Don't use this method!
 *      Use Connection connetion.disable();
    */
    public final void unconnect(Variable variable){
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        Iterator e=getParentModelComponent().getModelComponents().iterator();
        Connection c;
        Object o;
        while(e.hasNext()){
            o=e.next();
            if (o instanceof Connection){
                c=(Connection)o;
                if (c.getTarget()==variable && c.getDomain()==this){
                    if (c.isEnabled()){
                        c.disable();
                        getParentModelComponent().removeModelComponent(c);
                        return;
                    }
                }
            }
        }
    }

    /** Don't call this method directly, but use the Connection class instead.
    *  Sets the variable which defines through a chain of connects the value of this variable.
    */
    /* by af: new version for Connection class*/
    public final void setDefinedBy(Variable defining){
        if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
            getSimApplication().getIntegrator().stateEventOccurred = true;
            return;
        }
        if (defining==null) defining=this; /* should not be null */
        if (definedBy==defining) return;
        definedBy=defining;
        informOutgoingCouplers();
    }

    protected final void informOutgoingCouplers(){
        Iterator e=outCouplers.iterator();
        Coupler c;
        while(e.hasNext()){
            c=(Coupler)e.next();
            c.definedByChange(this);
        }
    }


    /** Gets the variable which defines through a chain of connects the value of this variable.*/
    public final Variable getDefinedBy() {
        return definedBy;
    }


// interface InputEventListener  -------------------------------------------------------------------

    Vector inputEventComponents = new Vector();

    /** Adds a InputEventComponent. Is called from addInputEventListener of InputEventComponent. */
    public void addInputEventComponent (SimEventComponent c){
        inputEventComponents.add(c);
    }

    /** Removes a InputEventComponent. Is called from removeInputEventListener of InputEventComponent.  */
    public void removeInputEventComponent (SimEventComponent c){
        inputEventComponents.remove(c);
    }

    /** Adds a SimEventComponent this listener listens to. Is called from addSimEventListener of SimEventComponent. */
    public void addSimEventComponent (SimEventComponent c){
    }

    /** Removes a SimEventComponent this listener listens to. Is called from removeSimEventListener of SimEventComponent.  */
    public void removeSimEventComponent (SimEventComponent c){
    }

    /** Reacts to SimEvents from SimEventComponents.
    * This is the event method for reacting to SimEvents
    */
    public void processSimEvent (SimEvent evt){
    }

}  // end AbstractVariable =========================================================================

