/*
 *  SortedSetVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.lang.*;
import java.util.*;

/**
 * Abstract class for all Variables which function like a SortedSet.
 */
abstract public class SortedSetVariable
   extends mosaic.sim.kernel.variable.SetVariable //++ not compatible with Component: implements SortedSet
{

    /** Standard comparator for FIFO sorting */
    public final FIFOComparator FIFO = new FIFOComparator();

    /** Standard comparator for LIFO sorting */
    public final LIFOComparator LIFO = new LIFOComparator();

    /** sorting criteria */
    Comparator sortingCriteria = FIFO;

    /** Nr for marking the insertion times */
    long insertionNr = 0;
    /** Map to remember the insertion times of the objects */
    protected TreeMap insertionTimeTable = new TreeMap();


// Constructors ------------------------------------------------------------------------------------

  /** Constructor for SortedSetVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type. */
  public SortedSetVariable(String name, int type, String unit, String quantity, String displayUnit) {
      super(name, type, unit, quantity, displayUnit);
   }


// sorting criteria --------------------------------------------------------------------------------

    /** Sets a new sorting criteria for the sorted set*/
    abstract public void setSorting(Comparator comparator);

    /** Gets the sorting criteria for the sorted set*/
    abstract public Comparator getSorting();

// standard sorting criteria -----------------------------------------------------------------------

    class FIFOComparator implements Comparator {

        public int compare(Object o1, Object o2) {
            Long o1Time = ((Long)insertionTimeTable.get(o1));
            Long o2Time = ((Long)insertionTimeTable.get(o2));
            if (o1Time != null && o2Time != null) {
                return (o1Time.compareTo(o2Time));
            } else {
                return ((java.lang.Comparable) o1).compareTo(o2);
            }
        }
    }

    class LIFOComparator implements Comparator {

        public int compare(Object o1, Object o2) {
            Long o1Time = ((Long)insertionTimeTable.get(o1));
            Long o2Time = ((Long)insertionTimeTable.get(o2));
            if (o1Time != null && o2Time != null) {
                return (o2Time.compareTo(o1Time));
            } else {
                return ((java.lang.Comparable) o1).compareTo(o2);
            }
        }
    }

// value as SortedSet ------------------------------------------------------------------------------

// getValueAsSortedSet
    /** Gets the value as Set object */
    abstract public SortedSet getValueAsSortedSet ();


// Collection interface implementation -------------------------------------------------------------
// All requests are forwarded to the value value.
// In all methods which will change the value, a variableChange is signaled.

    public boolean _add(Object o) {
        insertionTimeTable.put(o, new Long(insertionNr));
        if (getValueAsSortedSet().add(o)) {
            insertionNr++;
            signalVariableChange();
            return true;
        } else {
            insertionTimeTable.remove(o);
            return false;
        }
    }

    public boolean _remove(Object o) {
        if (getValueAsSortedSet().remove(o)) {
            insertionTimeTable.remove(o);
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }


    public boolean _containsAll(Collection c) {
        return getValueAsSortedSet().containsAll(c);
    }

    public boolean _addAll(Collection c) {
        Iterator i1 = c.iterator();
        while (i1.hasNext()) {
            insertionTimeTable.put(i1.next(), new Long(insertionNr));
            insertionNr++;
        }
        if (getValueAsSortedSet().addAll(c)) {
            signalVariableChange();
            return true;
        } else {
            Iterator i2 = c.iterator();
            while (i2.hasNext()) {
                insertionTimeTable.remove(i2.next());
                insertionNr--;
            }
            return false;
        }
    }

    public boolean _removeAll(Collection c) {
        if (getValueAsSortedSet().removeAll(c)) {
            Iterator i = c.iterator();
            while (i.hasNext()) {
                insertionTimeTable.remove(i.next());
            }
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public boolean _retainAll(Collection c) {
        if (getValueAsSortedSet().retainAll(c)) {
            Iterator i = insertionTimeTable.keySet().iterator();
            while (i.hasNext()) {
                Object key = i.next();
                if (! c.contains(key)) {
                    insertionTimeTable.remove(key);
                }
            }
            signalVariableChange();
            return true;
        } else {
            return false;
        }
    }

    public void _clear(){
        getValueAsSortedSet().clear();
        insertionTimeTable.clear();
        signalVariableChange();
    }

// Sorted Set interface implementation -------------------------------------------------------------
// All requests are forwarded to the value set.
// In all methods which will change the value, a variableChange is signaled.

    public Comparator _comparator() {
        return getValueAsSortedSet().comparator();
    }

    public SortedSet _subSet(Object fromElement, Object toElement) {
        return getValueAsSortedSet().subSet(fromElement, toElement);
    }

    public SortedSet _headSet(Object toElement) {
        return getValueAsSortedSet().headSet(toElement);
    }

    public SortedSet _tailSet(Object fromElement) {
        return getValueAsSortedSet().tailSet(fromElement);
    }

    public Object _first() {
        return getValueAsSortedSet().first();
    }

    public Object _last() {
        return getValueAsSortedSet().last();
    }

}

