/**
 * Project mosaic
 *
 * Variable.java
 *
 * copyright 2000, JKU Linz, Austria
 *
 */

package mosaic.sim.kernel.variable;

import java.util.*;
import mosaic.sim.kernel.simevent.*;

/**
* Interface for all model components which are variables, i.e., which can store values and
* fire VariableChangeEvents when they are set. Also variables can be connected, have a type
* (INPUT; OUTPUT, STATE, CONSTANT, and VARIABLE), a unit, , a display unit and a quantity.
* <p>
* Is implemented by abstract class AbstractVariable from which special variables which store
* values of different type (e.g. IntVariable) are derived.
*/
public interface Variable extends  mosaic.sim.kernel.variable.VariableChangeComponent, InputEventListener  {

// constants  --------------------------------------------------------------------------------------

   /** Constant to specify that the Variable is a input model variable */
   public static final int INPUT      = 0x0002;
   /** Constant to specify that the Variable is a output model variable */
   public static final int OUTPUT     = 0x0004;
   /** Constant to specify that the Variable is a model constant */
   public static final int CONSTANT   = 0x0008;
   /** Constant to specify that the Variable of a auxiliary model variable */
   public static final int VARIABLE   = 0x0010;
   /** Constant to specify that the Variable is a state model variable */
   public static final int STATE      = 0x0001;


// variable type -----------------------------------------------------------------------------------

   /** Tests if the variable is a STATE. */
   public boolean isTypeState();

   /** Tests if the variable is a INPUT. */
   public boolean isTypeInput();

   /** Tests if the variable is a OUTPUT. */
   public boolean isTypeOutput();

   /** Tests if the variable is a CONSTANT. */
   public boolean isTypeConstant();

   /** Tests if the variable is a VARIABLE. */
   public boolean isTypeVariable();

   /** Sets the type of the variable to STATE. */
   public void setTypeState();

   /** Sets the type of the variable to INPUT. */
   public void setTypeInput();

   /** Sets the type of the variable to OUTPUT. */
   public void setTypeOutput();

   /** Sets the type of the variable to CONSTANT. */
   public void setTypeConstant();

   /** Sets the type of the variable to VARIABLE. */
   public void setTypeVariable();


// properties --------------------------------------------------------------------------------------

   /** Gets the unit of the variable. */
   public String getUnit();

   /** Gets the quantity of the variable. */
   public String getQuantity();

   /** Gets the display unit of the variable. */
   public String getDisplayUnit();

// couplings    ------------------------------------------------------------------------------------

   /** Adds a coupler that listens to this variable. */
   public void addOutCoupler(Coupler c);

   /** Removes a coupler that listens to this variable. */
   public void removeOutCoupler(Coupler c);

   /** Adds a coupler that sets this variable. To remove, set to null*/
   public void setInCoupler(Coupler c)throws mosaic.sim.kernel.variable.VariableException;

   /** Returns the ingoing coupler. */
   public Coupler getInCoupler();

   /** Returns all listening couplers. */
   public List getOutCouplers();

// check!   public void informOutgoingCouplers();

  /** Gets the variable which defines through a chain of connects the value of this variable.*/
   public Variable getDefinedBy();

   /** Sets the variable which defines through a chain of connects the value of this variable.*/
   public void setDefinedBy(Variable definedBy);


// signal varible change ---------------------------------------------------------------------------

   /** signals the change of the variable to the listeners. */
   public void signalVariableChange(); 

}  // end Variable =================================================================================

