/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: ConstantPLSynapse.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.8 $
 *  Last changed at: $Date: 2002/05/16 20:43:26 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

/** This class represents a synapse using piecewiese linear functions, but
 *  with a constant shape for all synapse objects in the simulation.
 */
public class ConstantPLSynapse extends PLSynapse {
	/** The global piecewise linear function. */
	protected static PLFunction response;

	/** Creates a synapse object and associates the pre- and postsynaptic
	 *  neurons with it.
	 *  @param pre The NeuronOutput object that will send SpikeEvents to this
	 *             synapse.
	 *  @param post The Neuron object that will receive the forwarded spike
	 *              events from this synapse.
	 */
	public ConstantPLSynapse(NeuronOutput pre, Neuron post) {
		super(pre, post);
	}

	/** Creates a synapse object, associates the pre- and postsynaptic
	 *  neurons with it and initializes the synaptic weight.
	 *  @param pre The NeuronOutput object that will send SpikeEvents to this
	 *             synapse.
	 *  @param post The Neuron object that will receive the forwarded spike
	 *              events from this synapse.
	 *  @param weight The initial synaptic weight value.
	 */
	public ConstantPLSynapse(NeuronOutput pre, Neuron post, float weight) {
		super(pre, post, weight);
	}

	/** Creates a synapse object, associates the pre- and postsynaptic
	 *  neurons with it and initializes the synaptic weight and delay
	 *  @param pre The NeuronOutput object that will send SpikeEvents to this
	 *             synapse.
	 *  @param post The Neuron object that will receive the forwarded spike
	 *              events from this synapse.
	 *  @param weight The initial synaptic weight value.
	 *  @param delay The initial synaptic delay value.
	 */
	public ConstantPLSynapse(NeuronOutput pre, Neuron post,
						float weight, float delay) {
		super(pre, post, weight, delay);
	}

	/** Creates a synapse object, associates the pre- and postsynaptic
	 *  neurons with it and initializes the synaptic weight and delay
	 *  @param pre The NeuronOutput object that will send SpikeEvents to this
	 *             synapse.
	 *  @param post The Neuron object that will receive the forwarded spike
	 *              events from this synapse.
	 *  @param weight The initial synaptic weight value.
	 *  @param delay The initial synaptic delay value.
	 *  @param learningMethod The learning method used to change weight and
	 *                        delay.
	 */
	public ConstantPLSynapse(NeuronOutput pre, Neuron post,
						float weight, float delay,
						LearningMethod learningMethod) {
		super(pre, post, weight, delay, learningMethod);
	}

	/** Implementation of PLSynapse.getStandardResponse() */
	protected PLFunction getStandardResponse() {
		return response;
	}

	/** Implementation of PLSynapse.getResponse() */
	public static PLFunction getResponse() {
		return response;
	}

	/** Used to set the globally constant shape of the synapse response
	 *  function.
	 *  @see #response
	 */
	public static void setResponse(PLFunction response) {
		Debug.assertCond(response.gradients[response.gradients.length-1] == 0,
			"ConstantPLSynapse.setResponse",
			"Synaptic response function must terminate with a gradient of 0.");
		ConstantPLSynapse.response = response;
	}
}
