/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: ConstantPLThreshold.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.10 $
 *  Last changed at: $Date: 2002/05/16 20:43:26 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

/** This class represents a threshold using piecewiese linear functions, but
 *  with a constant shape for all threshold objects in the simulation.
 */
public class ConstantPLThreshold extends PLThreshold {
	/* The globally constant quiesent value of the threshold. */
	protected static float quiescentValue;

	/** The globally constant absolute refraction time of the threshold. */
	protected static float absoluteRefractionTime;

	/* The globally constant relative refraction function, piecewise linear. */
	protected static PLFunction relativeRefraction;

	/* The globally constant relative refraction function start value. */
	protected static float relativeRefractionStartValue;

	/** The globally constant relative refraction time, that is the time after
	 *  the quiescent value is reached again. This value is calculated by
	 *  calculateRelativeRefractionTime and does not need to be set.
	 */
	private static float relativeRefractionTime;

	/** Creates a ConstantPLThreshold object, but does no initialization. */
	public ConstantPLThreshold() {
	}

	/** Implementation of PLThreshold.getQuiescentValue(). */
	public float getQuiescentValue() {
		return quiescentValue;
	}

	/** Implementation of PLThreshold.getAbsoluteRefractionTime(). */
	public float getAbsoluteRefractionTime() {
		return absoluteRefractionTime;
	}

	/** Implementation of PLThreshold.getRelativeRefractionTime(). */
	public float getRelativeRefractionTime() {
		return relativeRefractionTime;
	}

	/** Implementation of PLThreshold.getRelativeRefraction(). */
	public PLFunction getRelativeRefraction() {
		return relativeRefraction;
	}

	/** Implementation of PLThreshold.getRelativeRefractionStartValue(). */
	public float getRelativeRefractionStartValue() {
		return relativeRefractionStartValue;
	}

	/*public static float getQuiescentValue() {
		return quiescentValue();
	}*/
	/** Set the globally constant quiescent value of the threshold.
	 *  @param value The new qiescent value
	 *  @see #quiescentValue
	 */
	public static void setQuiescentValue(float value) {
		quiescentValue = value;
	}

	/*public static float getAbsoluteRefractionTime() {
		return absoluteRefractionTime;
	}*/
	/** Set the globally constant absolute refraction time.
	 *  @param value The new absolute refraction time
	 *  @see #absoluteRefractionTime
	 */
	public static void setAbsoluteRefractionTime(float value) {
		absoluteRefractionTime = value;
	}

	/*public static float getRelativeRefractionStartValue() {
		return relativeRefractionStartValue;
	}*/
	/** Set the globally constant relative refraction start value.
	 *  @param value The new relative refraction start value
	 *  @see #relativeRefractionStartValue
	 */
	public static void setRelativeRefractionStartValue(float value) {
		relativeRefractionStartValue = value;
	}

	/** Set the globally constant relative refraction function.
	 *  @param function The new piecewise linear relative refraction function
	 *  @see #relativeRefraction
	 */
	public static void setRelativeRefraction(PLFunction function) {
		relativeRefraction = function;
	}

	/** Calculates the relative refraction time, that is the time until the
	 *  threshold reaches its quiescent value again. This method should be
	 *  called after all parameters have been set.
	 */
	public static void calculateRelativeRefractionTime() {
		/* calculate the relative refraction time (the time until the threshold
		   value is again equal to its quiescent value after the neuron has entered
		   the relative refraction phase) */
		if (relativeRefraction.gradients.length > 0) {
			relativeRefractionTime = relativeRefractionStartValue - quiescentValue;
			int i;
			for (i=0; i<relativeRefraction.gradients.length-1; i++)
				relativeRefractionTime += relativeRefraction.gradients[i] *
				(relativeRefraction.gradientTimes[i+1] -
				relativeRefraction.gradientTimes[i]);
			relativeRefractionTime = (float) relativeRefraction.gradientTimes[i] -
				(relativeRefractionTime / (float) relativeRefraction.gradients[i]);
			Debug.assertCond(relativeRefractionTime >= 0 &&
				relativeRefractionTime >=
				relativeRefraction.gradientTimes[
				relativeRefraction.gradientTimes.length-1],
				"ConstantPLThreshold.calculateRelativeRefractionTime",
				"relative refraction time can't be less than the last function " +
				"point in the relative refraction function.");
		}
		else
			relativeRefractionTime = 0;
	}
}
