/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: Neuron.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.13 $
 *  Last changed at: $Date: 2002/05/16 20:43:26 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

import java.util.*;
import mosaic.sim.kernel.simevent.*;
import mosaic.sim.kernel.variable.*;

/** This class represents a neuron, which is able to fire spike events (thus
 *  it implements the NeuronOutput interface) and receive spikes indirectly via
 *  Synapses.
 */
public class Neuron extends NeuronOutputSupport
					implements VariableChangeListener {
	/** This is the list of Synapses which receive spike events from another
	 *  NeuronOutput object and forward the event to this Neuron object.
	 *  It should only contain Synapse objects, nothing else.
	 */
	protected ArrayList inputs;

	/** The inner potential of this neuron. It contains the expected future
	 *  development of the neuron potential and is - in conjunction with the
	 *  threshold - used to compute the expected next fire time of this Neuron.
	 *  When one of the input synapses of this Neuron receives a spike event,
	 *  the potential will be updated to reflect this neuron's response to the
	 *  incoming spike and a new fire time will be computed.
	 */
	protected Potential potential;

	/** The inner threshold of this neuron. When the potential reaches a value
	 *  that is equal to or greater than the current threshold value, then this
	 *  neuron will fire.
	 */
	protected Threshold threshold;

	/** This is a master copy of the synapses that will be used for connecting
	 *  neurons with each other. The actual synapses between the neurons will
	 *  be cloned from this one when the addInput method is used to add a
	 *  NeuronOutput object to the input list of this neuron.
	 */
	protected Synapse masterSynapse;

	/** This method calculates the next firing time of this neuron (by using
	 *  the method computeTimeOfFire) and schedules the neuron for activation
	 *  at this time if a firing occurs or deactivates it if no firing will
	 *  occur.
	 *  @see #computeTimeOfFire
	 */
	protected void scheduleNextFiring() {
		double dt = computeTimeOfFire();
		if (dt == Double.POSITIVE_INFINITY) {
			Debug.output(Debug.LEVEL_INFO, "deactivating Neuron " + label + "\n");
			passivate();
		}
		else {
			Debug.output(Debug.LEVEL_INFO, "scheduling Neuron " + label +
				" for activation in " + dt + "\n");
			activateAt(dt);
		}
	}

	/** Default constructor with no arguments.
	 *  It only initializes the input and output lists with emtpy ArrayList
	 *  objects.
	 */
	public Neuron () {
		inputs = new ArrayList();
		outputs = new ArrayList();
		threshold = null;
		potential = null;
	}

	/** This method is called by the synapses from the inputs list whenever
	 *  they receive a spike event from their sender object.
	 *  It updates the potential according to the response parameters from
	 *  the synapse that called this method and (re-)computes the next fire
	 *  time of this neuron.
	 */
	public void spikeEventFrom(Synapse sy) {
		Debug.assertCond(sy != null, "Neuron.spikeEventFrom",
			"given Synapse object can not be null");
		Debug.output(Debug.LEVEL_INFO, "Neuron.spikeEventFrom for " +
			label + " at time "	+ getTime() + "\n");
		potential.mergeSynapseResponse(sy.getResponseParameters());
		scheduleNextFiring();
	}

	/** Computes the next fire time according to the current potential and
	 *  threshold parameters.
	 *  The actual computation is performed by the threshold object, using the
	 *  current potential object.
	 *  @return Returns either the absolute time of the next firing event or
	 *          POSITIVE_INFINITY when no firing will occur with the current
	 *          potential and threshold functions (when there is no
	 *          intersection).
	 */
	public double computeTimeOfFire() {
		return threshold.calculateNextFireTime(potential);
	}

	/** This method is called by the simulation framework when a time event
	 *  occurs. It is used within the Neuron for firing. Thus the neuron fired
	 *  (it sends out SpikeEvents) when the time event occurs.
	 */
	public void processSimEvent(SimEvent e) {
		// this is a spike event where the neuron activates itself
		fireSpikeEvent();
		/* also notify the threshold object of the firing - it will probably go
		   into a refraction state */
		threshold.fired();
		// and all input synapses
		for (int i=0; i<inputs.size(); i++)
			((Synapse) inputs.get(i)).postNeuronFired();
		/* and schedule the next firing event (the neuron potential might
		   already have another intersection with the threshold function
		   because of the previously received post synaptic potentials */
		scheduleNextFiring();
	}

	/** This method is called by the simulation framework whenever a variable
	 *  that this object is listening on changes. It is used within the Neuron
	 *  for listening to changes on the Potential and the Threshold objects
	 *  associated with the Neuron. The implementation simply calls
	 *  scheduleNextFiring() because changes in the potential or the threshold
	 *  objects might cause the next firing time to change.
	 *  @see #scheduleNextFiring
	 */
	public void variableChange (mosaic.sim.kernel.variable.VariableChangeEvent e) {
		/* Since this method is called when either the potential or the
		   threshold object changes the internal state, it might be necessary
		   to schedule a firing event (e.g. the threshold object might have
		   returned from its refraction state and now there is an intersection
		   with the potential function again). */
		scheduleNextFiring();
	}

	/**************************************************************************
	 *  Set / Get Methods */
	/** Returns the current neuron potential.
	 */
	public double getCurrentPotential(){
		return potential.getCurrentPotential();
	}

	/** Returns the threshold object.
	 */
	public Threshold getThreshold(){
		return threshold;
	}

	/** Sets the threshold object.
	 */
	public void setThreshold(Threshold threshold){
		/* If another threshold object was already set, then unregister this
		   neuron object (as VariableChangeListener) from the threshold object
		   before losing the reference. */
		if (threshold != null)
			threshold.removeVariableChangeListener(this);
		this.threshold = threshold;
		// and register this object with the new one
		if (threshold != null)
			threshold.addVariableChangeListener(this);
	}

	/** Returns the potential object.
	 */
	public Potential getPotential() {
		return potential;
	}

	/** Sets the potential object.
	 */
	 public void setPotential(Potential potential) {
		// the same goes here, see setThreshold
		if (potential != null)
			potential.removeVariableChangeListener(this);
		this.potential = potential;
		if (potential != null)
			potential.addVariableChangeListener(this);
	 }

	/** Returns the list of currently registerd input synapses.
	 */
	public Synapse[] getInputSynapses() {
		Synapse[] synapses = new Synapse[inputs.size()];
		for (int i=0; i<inputs.size(); i++)
			synapses[i] = (Synapse) inputs.get(i);

		return synapses;
	}

	/** Set the list of registered input synapses. This removes all synapse
	 *  objects that were registered before, so this method should only be used
	 *  during initialization.
	 *  For adding and removing synapses without removing the other registered
	 *  objects, use the methods addInputSynapse / removeInputSynapse or
	 *  addInput / removeInput.
	 *  @see #addInputSynapse
	 *  @see #removeInputSynapse
	 *  @see #addInput
	 *  @see #removeInput
	 */
	public void setInputSynapses(Synapse[] synapses) {
		Debug.assertCond(synapses != null, "Neuron.setInputSynapses",
			"synapses can not be null");
		inputs = new ArrayList();
		for (int i=0; i<synapses.length; i++)
			inputs.add(synapses[i]);
	}

	/** Adds a single synapse object to the list of input synapses.
	 */
	public void addInputSynapse(Synapse synapse) {
		inputs.add(synapse);
	}

	/** Removes a single synapse object from the list of input synapses.
	 *  @return true when the given object was found and could be removed and
	 *           false when the given object could not be found in the current
	 *           list of registered input synapses.
	 */
	public boolean removeInputSynapse(Synapse synapse) {
		return inputs.remove(synapse);
	}

	/** This method adds another input to this Neuron, but the given input
	 *  object does not have to be a Synpase, it just has to be some object
	 *  implementing NeuronOutput. Because Neurons depend on Synapses as their
	 *  input, this method will create the needed synapse object by cloning it
	 *  from the masterSynapse object, which has to be set beforehand.
	 *  Please note that this method is only suited for easy construction of
	 *  the neural network structure, it has to be assured that weight, delay
	 *  and other variables are set before starting the simulation.
	 *  @see NeuronOutput
	 *  @see #masterSynapse
	 *  @see #addInputSynapse
	 */
	public void addInput(NeuronOutput sender) {
		/* Instead of creating a new Synapse object, clone the master synapse,
		   so that it works with different types of synapses. */
		Debug.assertCond(masterSynapse != null, "Neuron.addInput",
			"masterSynapse needed for cloning new snyapses, can not be null");
		Synapse newSynapse = (Synapse) masterSynapse.clone();
		newSynapse.setSender(sender);
		newSynapse.setReceiver(this);
		addInputSynapse(newSynapse);
		/* Also register the newly created synapse with the sender to complete
		   the (bidirectional) connection. */
		sender.addOutput(newSynapse);
	}

	/** Removes a NeuronOutput object from the input list of this neuron. This
	 *  method tries to locate the synapse object that connects the given object
	 *  with this neuron and removes it from the list of input synapse. It also
	 *  unregisters the synapse object from the given NeuronOutput object.
	 *  @return true when the given object was found and could be removed and
	 *           false when no synapse object could be found in the input list
	 *           that connects the given object with this neuron.
	 */
	public boolean removeInput(NeuronOutput sender) {
		// try to find a synapse that has pre=sender and post=this
		Synapse foundSynapse = null;
		for (int i=0; i<inputs.size() && foundSynapse == null; i++) {
			if (inputs.get(i) instanceof Synapse &&
				((Synapse) inputs.get(i)).getSender() == sender &&
				((Synapse) inputs.get(i)).getReceiver() == this)
				foundSynapse = (Synapse) inputs.get(i);
		}
		if (foundSynapse != null) {
			// unregister the synapse from the sender object before removing it
			sender.removeOutput(foundSynapse);
			return removeInputSynapse(foundSynapse);
		}
		else
			// not found
			return false;
	}

	/** Returns the master synapse object used for creating new synapse objects
	 *  on the fly in the method addInput().
	 */
	public Synapse getMasterSynapse() {
		return masterSynapse;
	}

	/** Sets the master synapse object used for creating new synapse objects
	 *  on the fly in the method addInput().
	 */
	public void setMasterSynapse(Synapse master) {
		masterSynapse = master;
	}
}
