/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: PLThreshold.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.16 $
 *  Last changed at: $Date: 2002/05/16 20:43:26 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

import mosaic.sim.kernel.simevent.*;

/** This class implements a threshold with piecewise linear functions. */
public abstract class PLThreshold extends Threshold {
	/** The quiescent state means that the threshold is constant (not changing
	 *  over the time as long as the associated neuron does not fire).
	 */
	public final static int STATE_QUIESCENT = 1;
	/** The absolute refraction state means that the threshold is infinite (the
	 *  associated neuron can not fire in any case).
	 */
	public final static int STATE_ABSOLUTE_REFRACTION = 2;
	/** The relative refraction state means that the threshold is higher than
	 *  the quiescent value, but approaches the quiescent value.
	 */
	public final static int STATE_RELATIVE_REFRACTION = 3;

	/** The simulation time when the associated neuron fired the last time. */
	protected double lastFireTime;
	/** The current state of the neuron, can be one of STATE_*. */
	protected int state;
	/** The previous state of the neuron, can be one of STATE_*. */
	protected int lastState;

	protected double calculateIntersectionPoint(
			  PLFunction f1, double startValue1, double startTime1, int numSegments1,
			  PLFunction f2, double startValue2, double startTime2, int numSegments2) {
		int i1, i2;
		double x1 = startValue1, x2 = startValue2, k1, k2, t1, t2,
			   t, fireTime = 0;
		boolean foundIntersectionWithin = false, foundIntersectionBefore = false;
		/* skip past segments of both functions because firing can only occur
		   in the future */
		PLFunction.FloatIntPair ret1 = f1.getFinishedSegments(
			startValue1, numSegments1, getTime() - startTime1, false);
		PLFunction.FloatIntPair ret2 = f2.getFinishedSegments(
			startValue2, numSegments2, getTime() - startTime2, false);
		i1 = ret1.i;
		x1 = ret1.f;
		i2 = ret2.i;
		x2 = ret2.f;

		// check if the potential reaches the theshold
		/* This is done as follows: Always calculate the intersection
		   between segments of the potential and the threshold function -
		   if the intersection is outside the scope of the current segments,
		   check the next segments. */
		while (! foundIntersectionWithin &&
			  i1 < numSegments1-1 &&
			  i2 < numSegments2-1) {
			// calculate the intersection between the segments
			k1 = f1.gradients[i1];
			k2 = f2.gradients[i2];
			t1 = f1.gradientTimes[i1] + startTime1;
			t2 = f2.gradientTimes[i2] + startTime2;
			if (k1 != k2) {
			   t = (x1 - x2 - k1*t1 + k2*t2) / (k2 - k1);
			   if (t < t1 || t < t2) {
				   // the intersection is before the current segments
				   /* This can only happen for the very first segments in this
					  loop because for all other than the first segments, the
					  next if would have had matched. */
					foundIntersectionBefore = true;
			   }
			   else if (t <= f1.gradientTimes[i1+1] ||
					t <= f2.gradientTimes[i2+1]) {
					// the intersection is within the segments
					fireTime = t;
					foundIntersectionWithin = true;
			   }
			   // else the intersection is after the current segments
			}
			/* else no intersection possible - the gradients are equals and
			   hence the segments are parallel */

			// now try the next segment
			if (f1.gradientTimes[i1+1] + startTime1 >
			   f2.gradientTimes[i2+1] + startTime2) {
				i2++;
			}
			else if (f1.gradientTimes[i1+1] + startTime1 <
			   f2.gradientTimes[i2+1] + startTime2) {
				i1++;
			}
			else {
				i1++; i2++;
			}
		}

		if (foundIntersectionWithin) {
			// already found an intersection, no need to search further
			Debug.assertCond(!Double.isInfinite(fireTime),
				"PLThreshold.calculateIntersectionPoint",
				"Invalid result: calulation resulted in infinite value.");
			Debug.assertCond(!Double.isNaN(fireTime),
				"PLThreshold.calculateIntersectionPoint",
				"Invalid result: calulation resulted in NaN value.");
			return fireTime;
		}
		else {
			if (foundIntersectionBefore) {
				/* An intersection point has been found that is before of at
				   least one of the starting times of the given functions. */
				return Double.NEGATIVE_INFINITY;
			}
			else {
				/* No intersection within the functions - just
				   return POSITIVE_INFINITY since this method will be called
				   again when the threshold switches to quiescent state again
				   (and intersection occuring within this period will be
				   calculated then. */
				return Double.POSITIVE_INFINITY;
			}
		}
	}

	/** Creates a PLThreshold object and sets its state to STATE_QUIESCENT. */
	public PLThreshold() {
		state = lastState = STATE_QUIESCENT;
	}

	/** Returns the next fire time of the associated neuron. That is the first
	 *  possible intersection of the current threshold function with the given
	 *  potential function.
	 *  @param potential The potential function to compute the intersection
	 *                   with.
	 */
	public double calculateNextFireTime(Potential potential) {
		// a PLThreshold can only compute with a PLPotential
		Debug.assertCond(potential instanceof PLPotential,
			"PLThreshold.calculateNextFireTime",
			"potential must be of type PLPotential");
		PLPotential plpotential = (PLPotential) potential;
		double fireTime;

		if (state == STATE_ABSOLUTE_REFRACTION) {
			Debug.output(Debug.LEVEL_INFO, "Calculating next fire time for threshold object " + getName() + " in  absolute refraction state: POSITIVE_INFINITY\n");
			return Double.POSITIVE_INFINITY;
		}
		else if (state == STATE_RELATIVE_REFRACTION) {
			fireTime = calculateIntersectionPoint(
				   plpotential.getChanges(), 0.0, 0.0, plpotential.getNumSegments(),
				   getRelativeRefraction(), getRelativeRefractionStartValue(),
				   lastFireTime + getAbsoluteRefractionTime(),
				   getRelativeRefraction().gradients.length);
			if (fireTime == Double.NEGATIVE_INFINITY) {
				/* This can only happen for the first segments - when the
				   threshold switches to relative refraction and the
				   potential is higher then the threshold at this point
				   - just fire immediately. */
				System.out.println("Found intersection before current time at " +
					fireTime + ", setting current simulation time now.");
				fireTime = getTime();
			}
			Debug.output(Debug.LEVEL_INFO, "Calculating next fire time for threshold object " + getName() + " in relative refraction state: " + fireTime);
			return fireTime;
		} else {
			Debug.assertCond(state == STATE_QUIESCENT,
				"PLThreshold.calculateNextFireTime", "invalid state");

			Debug.output(Debug.LEVEL_INFO, "Calculating next fire time for threshold object " + getName() + " in quiescent state. Threshold is " +
				getQuiescentValue() + ", next fire time is ");

			PLFunction tempThreshold = new PLFunction(2);
			tempThreshold.gradients[0] = 0;
			tempThreshold.gradientTimes[0] = 0;
			tempThreshold.gradients[1] = 0;
			tempThreshold.gradientTimes[1] = Double.POSITIVE_INFINITY;
			fireTime = calculateIntersectionPoint(
				   plpotential.getChanges(), 0.0, 0.0, plpotential.getNumSegments(),
				   tempThreshold, getQuiescentValue(), getTime(), 2);
			if (fireTime == Double.NEGATIVE_INFINITY) {
				/* This can only happen for the first segments - when the
				   threshold switches to relative refraction and the
				   potential is higher then the threshold at this point
				   - just fire immediately. */
				System.out.println("Found intersection before current time at " +
					fireTime + ", setting current simulation time now.");
				fireTime = getTime();
			}

			Debug.output(Debug.LEVEL_INFO, fireTime + "\n");
			return fireTime;
		}
	}

	/** This method is called by the associated neuron object whenever it
	 *  fires.
	 *  The threshold then switches to STATE_ABSOLUTE_REFRACTION and activates
	 *  a timer to switch to STATE_RELATIVE_REFRACTION after the absolute
	 *  refraction time.
	 */
	public void fired() {
		Debug.output(Debug.LEVEL_INFO, "Threshold object " + getName() + " going to absolute refraction phase\n");
		lastState = state;
		state = STATE_ABSOLUTE_REFRACTION;
		lastFireTime = getTime();
		activateIn(getAbsoluteRefractionTime());

		// notify all listeners that this object has changed its internal state
		signalVariableChange();
	}

	/** This method is called by the simulation framework whenever a time event
	 *  occurs. It is used within PLThreshold for state internal transitions.
	 */
	public void processSimEvent(SimEvent e) {
		if (state == STATE_ABSOLUTE_REFRACTION &&
				getTime() >= lastFireTime + getAbsoluteRefractionTime()) {
			if (getRelativeRefractionTime() > 0) {
				Debug.output(Debug.LEVEL_INFO, "Threshold object " + getName() + " going to relative refraction phase\n");
				lastState = state;
				state = STATE_RELATIVE_REFRACTION;
				activateIn(getRelativeRefractionTime());
			}
			else {
				Debug.output(Debug.LEVEL_INFO, "Threshold object " + getName() + " going to quiescent phase\n");
				lastState = state;
				state = STATE_QUIESCENT;
			}
		}
		else if (state == STATE_RELATIVE_REFRACTION &&
				getTime() >= lastFireTime + getAbsoluteRefractionTime() +
				getRelativeRefractionTime()) {
			Debug.output(Debug.LEVEL_INFO, "Threshold object " + getName() + " going to quiescent phase\n");
			lastState = state;
			state = STATE_QUIESCENT;
		}

		// notify all listeners that this object has changed its internal state
		signalVariableChange();
	}

	/** Returns the current internal state.
	 *  @see #state
	 */
	public int getState() {
		return state;
	}

	/** Returns the last fire time of the associated neuron.
	 *  @see #lastFireTime
	 */
	public double getLastFireTime() {
		return lastFireTime;
	}

	/** Compute the current Neuron threshold from the gradient changes.
	 */
	public double getCurrentThreshold() {
		// this is dependent on the current state
		switch (state) {
			case STATE_QUIESCENT:
				return getQuiescentValue();
			case STATE_ABSOLUTE_REFRACTION:
				return Double.POSITIVE_INFINITY;
			case STATE_RELATIVE_REFRACTION:
				throw new UnsupportedOperationException();
			default:
				Debug.assertCond(false, "PLThreshold.getCurrentPotential",
					"unknown state");
				return 0;
		}
	}

	/** Returns the quiescent value. This method must be overridden in
	 *  subclasses.
	 */
	public abstract float getQuiescentValue();

	/** Returns the absolute refraction time. This method must be overridden in
	 *  subclasses.
	 */
	public abstract float getAbsoluteRefractionTime();

	/** Returns the relative refraction time. This method must be overridden in
	 *  subclasses.
	 */
	public abstract float getRelativeRefractionTime();

	/** Returns the relative refraction function. This method must be
	 *  overridden in subclasses.
	 */
	public abstract PLFunction getRelativeRefraction();

	/** Returns the start value of the relative refraction function. This
	 *  method must be overridden in subclasses.
	 */
	public abstract float getRelativeRefractionStartValue();
}
