/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: TemporalCodedInput.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.8 $
 *  Last changed at: $Date: 2002/05/16 20:43:31 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron.netinputs;

import java.util.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;
import mosaic.sim.neuron.*;

/** This class implements a spike source using temporal coding. It is a
 *  converter between analog values at its input and spike events at its
 *  output.
 *  The coding is done as follows: The number of inputs (i.e. the length of the
 *  input vector) is equal to the number of outputs (inner NeuronOutput
 *  classes). The analog vector submitted to the class via the method
 *  enterVector() is converted to exactly one spike per output. The higher the
 *  analog input number, the earlier the spike on the associated output will
 *  be fired. Therefore the output associated to the analog input with the
 *  highest value will fire immediately, the other outputs will fire later,
 *  determined by "d * (maxValue - vector[i])".
 *  This class can optionally use a reference output that fires at the latest
 *  time, thus representing the analog input value "0" and marking the end of
 *  the spike series emitted by this class.
 */
public class TemporalCodedInput extends BasicModel implements VectorInputCoder {
   /** This inner class implements the spike outputs of the TemporalCodedInput
	*  class.
	*/
	protected class Output extends NeuronOutputSupport {
		/** Set to true, when this output has scheduled a fire event, but has
		 *  not fired until now (i.e. the current simulation time).
		 */
		protected boolean active = false;

		/** Schedule firing in the specified time.
		 *  @param delta The delay until this output should fire.
		 */
		public void fireIn(double delta) {
			System.out.println("TemporalCodedInput.Output.fireIn: scheduling " +
				"for firing in " + delta);
			activateIn(delta);
			active = true;
		}

		/** This method is called by the simulation framework whenever a time
		 *  event occurs. It is used within this class for executing the
		 *  schedules spike event.
		 */
		public void processSimEvent(SimEvent e) {
			fireSpikeEvent();
			active = false;
		}

		/** Returns true if this output is still waiting to fire its spike.
		 *  @see #active
		 */
		public boolean isActive() {
			return active;
		}
	}

	/** The list of outputs that used for emitting spike events. The number of
	 *  outputs is equal to the length of the input vector.
	 */
	protected ArrayList outputs = null;
	/** The reference output, used optionally. */
	protected Output reference = null;
	/** This value is uses for computing the delay between the first spike
	 *  event and following spikes depending on the difference in the input
	 *  values. It is just a multiplicator.
	 */
	protected double multiplicator;
	/** In this variable, the last input vector that was entered using this
	 *  object is stored.
	 *  @see #enterVector
	 */
	protected double[] lastVector;
	/** In this variable, the last (absolute) time of starting an input event
	 *  is stored (the simulation time when enterVector was called).
	 *  @see #enterVector
	 */
	 protected double lastStartTime;
	 /** In this variable, the end of the last input cycle (the absolute
	  *  simulation time marking the input value 0) is stored. If a reference
	  *  input is used, then this is the time that the input has fired or will
	  *  fire.
	  *  @see #enterVector
	  */
	  protected double lastZeroTime;

	/** Creates a new TemporalCodedInput object and initializes the outputs.
	 *  @param num The number of outputs, it must be equal to the length of the
	 *             analog input vectors that will be given to enterVector().
	 *  @param multiplicator Sets the proportionality factor d.
	 *  @param useReferenceOutput When set to true, the class uses a reference
	 *                            output representing the analog input value
	 *                            "0".
	 */
	public TemporalCodedInput(int num, double multiplicator,
		   boolean useReferenceOutput) {
		outputs = new ArrayList(num);
		for (int i=0; i<num; i++) {
			Output output = new Output();
			outputs.add(i, output);
			add(output);
		}
		if (useReferenceOutput)
			reference = new Output();
		this.multiplicator = multiplicator;
		lastVector = null;
		lastStartTime = 0.0;
		lastZeroTime = 0.0;
	}

	/** Enters a new analog input vector into the simulation. This method
	 *  computes the firing times of the spike events (one per output) and
	 *  schedules them for firing.
	 *
	 *  @param vector The input vector to be used, the length must be equal to
	 *                "num" given to the constructor.
	 */
	public void enterVector(double[] vector) throws CodingException {
		// remember the vector
		lastVector = vector;

		boolean netInputCompleted = true;
		double maxValue=0;
		for (int i=0; i<outputs.size(); i++)
			if (((Output) outputs.get(i)).isActive())
				netInputCompleted = false;
		if (reference != null && reference.isActive())
			netInputCompleted = false;

		if (!netInputCompleted)
			throw new CodingException("Current input cycle incomplete, but new input started.");

		if (vector.length != outputs.size())
			throw new CodingException("Input vector length does not match.");

		// remember the maximum value
		for (int i=0; i<vector.length; i++) {
			if (vector[i] < 0 && vector[i] != Double.NEGATIVE_INFINITY)
				throw new CodingException("Invalid (negative) input value.");
			if (vector[i] > maxValue)
				maxValue = vector[i];
		}

		System.out.print("TemporalCodedInput.enterVector: starting network " +
			"input with vector: ");
		for (int i=0; i<vector.length; i++)
			System.out.print(vector[i] + " ");
		System.out.println(", maximum value is " + maxValue);

		// remember the cycle start and end times
		lastStartTime = getTime();
		lastZeroTime = multiplicator * maxValue + getTime();
		if (reference != null)
			reference.fireIn(multiplicator * maxValue);
		for (int i=0; i<vector.length; i++) {
			// NEGATIVE_INFINITY as value means that the corresponding input
			// should never fire
			if (vector[i] != Double.NEGATIVE_INFINITY)
				((Output) outputs.get(i)).fireIn(multiplicator *
						  (maxValue - vector[i]));
		}
	}

	/** Implementation of VectorInput.getOutput. */
	public NeuronOutput getOutput(int index) {
		if (index >= 0 && index < outputs.size())
			return (Output) outputs.get(index);
		else
			throw new ArrayIndexOutOfBoundsException();
	}

	/** Returns the reference output. */
	public NeuronOutput getReferenceOutput() {
		if (reference != null)
			return reference;
		else
			throw new NullPointerException();
	}

	/** Returns the last input vector. */
	public double[] getLastVector() {
		return lastVector;
	}

	/** Returns the multiplicator. */
	public double getMultiplicator() {
		return multiplicator;
	}

	/** Returns the time when the last event was started. */
	public double getLastStartTime() {
		return lastStartTime;
	}

	/** Return the time when the last event should end (or has ended). */
	public double getLastZeroTime() {
		return lastZeroTime;
	}
}
