/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: CBN.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.6 $
 *  Last changed at: $Date: 2002/04/14 15:18:37 $
 *  Last changed by: $Author: Rene $
 *****************************************************************************/

package mosaic.sim.neuron.simulations;

import mosaic.sim.*;
import mosaic.sim.kernel.*;
import mosaic.sim.neuron.*;
import mosaic.sim.neuron.netinputs.*;
import mosaic.sim.neuron.visualisation.*;
import java.awt.*;
import javax.swing.*;

/* This simulation implements a CBN (cuneate based network) structure as
 * discribed in
 * Eduardo Sanchez, Manual Mucientes, Senen Barro: "A Cuneate-Based Network and
 * its Application as a Spatio-Temporal Filter in Mobile Robotics", proceedings
 * of IWANN2001. */

public class CBN extends SimApplication {

	public CBN() {
		SimpleMainFrame frame = new SimpleMainFrame(this);

// ========> create your panel here <==========================================//
		CBNPanel p=new CBNPanel();
		frame.setModelPanel(p);
// ============================================================================//

		frame.pack();
		frame.setVisible(true);
		frame.setTitle("Cuneate Based Nucleus simulation");
		reset();
	}

	public static void main(String[] args) {
		CBN app = new CBN();
	}

}

class CBNPanel extends ModelPanel {
	public static final int NUMBER_INPUTS = 9;
	public static final int NUMBER_OUTPUTS = 7;
	public static final int RECEPTIVE_FIELD_WIDTH = 1;
	public static final int LATERAL_INHIBITION_WIDTH = 3;

	/** Only synapses for inhibitive feedback do not seem to be enough. The
	 *  reaction is then still not temporally filtered.
	 */
	public static final boolean USE_INTERNEURON_SYNAPSES = false;
	public static final boolean USE_INTERNEURONS = true;

	public static final boolean SHOW_INTERNEURONS_FIRING = true;
	public static final boolean SHOW_NEURON_DETAILS = true;

	public static final double WEIGHT_INPUTS = 1.5;
	/** If this parameter is set to 0, the simulation gets a significant
	 *  speedup. This is because of optimisations in the handling of synapses
	 *  with a value of 0. Although they participate in the network structure
	 *  and receive the firing events, they will not cause much cost.
	 */
	public static final double WEIGHT_LATERAL = 0.05;
	public static final double WEIGHT_INTERNEURON_FORWARD = 1.0;
	public static final double WEIGHT_INTERNEURON_BACKWARD = 10.0;
	public static final double DELAY_INTERNEURON = 0.5;

	NeuronOutput[] inputs = new NeuronOutput[NUMBER_INPUTS];
	Neuron[] outputs = new Neuron[NUMBER_OUTPUTS];
	Neuron[] interneurons = new Neuron[NUMBER_OUTPUTS];

	ViewSynchronizer sync = new ViewSynchronizer();
	ScrollingView[] views;

	PLFunction synapseResponse;

	public CBNPanel() {
		try {
			// some global settings - thresholds
			ConstantPLThreshold.setAbsoluteRefractionTime(0.7f);
			ConstantPLThreshold.setQuiescentValue(5);
			ConstantPLThreshold.setRelativeRefractionStartValue(5);
			ConstantPLThreshold.setRelativeRefraction(new PLFunction(0));
			ConstantPLThreshold.calculateRelativeRefractionTime();
			// synaptic responses
			synapseResponse = new PLFunction(3);
			synapseResponse.gradients[0] = 3.5;
			synapseResponse.gradientTimes[0] = 0;
			synapseResponse.gradients[1] = -2;
			synapseResponse.gradientTimes[1] = 1;
			synapseResponse.gradients[2] = 0;
			synapseResponse.gradientTimes[2] = synapseResponse.getEndTime(0, 2);
			System.out.println("Synapse response function set to:");
			System.out.print(synapseResponse.toString());
			ConstantPLSynapse.setResponse(synapseResponse);

			// now create the inputs
			for (int i=0; i<NUMBER_INPUTS; i++) {
				//inputs[i] = new RateCodedInput(0.1 * (i+1));
				inputs[i] = new RateCodedInputSingle(i < NUMBER_INPUTS/3 ? 0.1 :
						  (i < NUMBER_INPUTS*2/3 ? 0.2 : 1.0));
				((RateCodedInputSingle) inputs[i]).setName("input" + i);
				((RateCodedInputSingle) inputs[i]).setLabel("input" + i);
			}
			// and all the neurons
			for (int i=0; i<NUMBER_OUTPUTS; i++) {
				outputs[i] = new Neuron();
				outputs[i].setName("output" + i);
				outputs[i].setLabel("output" + i);
				// don't forget to create the potentials and thresholds
				PLPotential op = new PLPotential(1);
				op.setName("potential output" + i);
				outputs[i].setPotential(op);
				ConstantPLThreshold ot = new ConstantPLThreshold();
				ot.setName("threshold output" + i);
				outputs[i].setThreshold(ot);

				if (USE_INTERNEURONS) {
					interneurons[i] = new Neuron();
					interneurons[i].setName("interneuron" + i);
					interneurons[i].setLabel("interneuron" + i);
					PLPotential in_p = new PLPotential(50);
					in_p.setName("potential interneuron" + i);
					interneurons[i].setPotential(in_p);
					ConstantPLThreshold in_t = new ConstantPLThreshold();
					in_t.setName("threshold interneuron" + i);
					interneurons[i].setThreshold(in_t);

					// and connect the interneuron with the output neuron
					ConstantPLSynapse sy_forward = new ConstantPLSynapse(
						outputs[i], interneurons[i],
						(float) WEIGHT_INTERNEURON_FORWARD, 0f);
					sy_forward.setName("synapse forward interneuron " + i);
					sy_forward.setLabel("synapse forward interneuron " + i);
					ConstantPLSynapse sy_backward = new ConstantPLSynapse(
						interneurons[i], outputs[i],
						(float) -WEIGHT_INTERNEURON_BACKWARD,
						(float) DELAY_INTERNEURON);
					sy_backward.setName("synapse backward interneuron " + i);
					sy_backward.setLabel("synapse backward interneuron " + i);
				}

				if (USE_INTERNEURON_SYNAPSES) {
					ConstantPLSynapse sy_interneuron = new ConstantPLSynapse(
						outputs[i], outputs[i],
						(float) -WEIGHT_INTERNEURON_BACKWARD,
						(float) DELAY_INTERNEURON);
					sy_interneuron.setName("synapse interneuron " + i);
					sy_interneuron.setLabel("synapse interneuron " + i);
				}
			}

			// create the lateral inhibitive connections in the output layer
			for (int i=0; i<NUMBER_OUTPUTS; i++) {
				for (int j=-LATERAL_INHIBITION_WIDTH;
					j<=LATERAL_INHIBITION_WIDTH; j++) {
					if (i+j >= 0 && i+j < NUMBER_OUTPUTS) {
						// this is the mexican hat function in some way...
						double weight;
						if (j==0)
						   weight = WEIGHT_LATERAL;
						else
							weight = -3 * WEIGHT_LATERAL;
						/*else
							weight = -(LATERAL_INHIBITION_WIDTH - Math.abs(j)) / LATERAL_INHIBITION_WIDTH;*/
						ConstantPLSynapse sy_lateral = new ConstantPLSynapse(
							outputs[i+j], outputs[i], (float)weight, 1);
						sy_lateral.setName("synapse lateral" + i+j + "->" + i);
						sy_lateral.setLabel("synapse lateral" + i+j + "->" + i);
					}
				}
			}

			// at last, the connections from the inputs to the outputs
			for (int i=0; i<NUMBER_OUTPUTS; i++) {
				for (int j=-RECEPTIVE_FIELD_WIDTH; j<=RECEPTIVE_FIELD_WIDTH;
					j++) {
					// here we use the sobel edge extractor as an example
					// WARNING - HACK !!!! the "+1" should go away an be more general !!
					if (i+j+1 >= 0 && i+j+1 < NUMBER_INPUTS) {
						double weight;
						if (j==0)
							weight = 2 * RECEPTIVE_FIELD_WIDTH * WEIGHT_INPUTS;
						else
							weight = -WEIGHT_INPUTS;
						ConstantPLSynapse sy_input = new ConstantPLSynapse(
							inputs[i+j+1], outputs[i], (float) weight, 0);
						sy_input.setName("synapse input" + i+j+1 + "->" + i);
						sy_input.setLabel("synapse input" + i+j+1 + "->" + i);
					}
				}
			}

			// at last build up the layout
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			views = new ScrollingView[NUMBER_INPUTS + NUMBER_OUTPUTS +
				  (SHOW_NEURON_DETAILS ? NUMBER_OUTPUTS : 0) +
				  (SHOW_INTERNEURONS_FIRING ? NUMBER_OUTPUTS : 0) +
				  (SHOW_INTERNEURONS_FIRING && SHOW_NEURON_DETAILS ?
					  NUMBER_OUTPUTS : 0)];
			for (int i=0; i<NUMBER_INPUTS; i++) {
				add((RateCodedInputSingle) inputs[i]);
				views[i] = new SpikeView("spikeview input" + i);
				inputs[i].addOutput((SpikeView) views[i]);
			}
			for (int i=0; i<NUMBER_OUTPUTS; i++) {
				add(outputs[i]);
				if (USE_INTERNEURONS)
				   add(interneurons[i]);
				views[NUMBER_INPUTS+i] = new SpikeView("spikeview output" + i);
				outputs[i].addOutput((SpikeView) views[NUMBER_INPUTS+i]);
				if (SHOW_NEURON_DETAILS) {
					views[NUMBER_INPUTS+NUMBER_OUTPUTS+i] =
						new PLNeuronInternalView("detailsview output" + i, true, true);
					outputs[i].getPotential().addVariableChangeListener(
						(PLNeuronInternalView) views[NUMBER_INPUTS+NUMBER_OUTPUTS+i]);
					outputs[i].getThreshold().addVariableChangeListener(
						(PLNeuronInternalView) views[NUMBER_INPUTS+NUMBER_OUTPUTS+i]);
				}

				if (SHOW_INTERNEURONS_FIRING) {
					int j = NUMBER_INPUTS+NUMBER_OUTPUTS+i +
						(SHOW_NEURON_DETAILS ? NUMBER_OUTPUTS : 0);
					views[j] = new SpikeView("spikeview interneuron" + i);
					interneurons[i].addOutput((SpikeView) views[j]);
					if (SHOW_NEURON_DETAILS) {
						views[j+NUMBER_OUTPUTS] =
							new PLNeuronInternalView("detailsview output" + i, true, true);
						interneurons[i].getPotential().addVariableChangeListener(
							(PLNeuronInternalView) views[j+NUMBER_OUTPUTS]);
						interneurons[i].getThreshold().addVariableChangeListener(
							(PLNeuronInternalView) views[j+NUMBER_OUTPUTS]);
					}
				}
			}

			for (int i=0; i<views.length; i++) {
				views[i].setName("v" + (i+1));
				views[i].setTMax(50);
				views[i].setCurrentTimePosition(0.5);
				sync.addView(views[i]);
				this.add(views[i]);
				if (views[i] instanceof PLNeuronInternalView) {
				   views[i].setYMin(-1);
				   views[i].setYMax(15);
				}
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void init() {
		// initialize the simulation
		for (int i=0; i<inputs.length; i++)
			((RateCodedInputSingle) inputs[i]).init();
	}

	public void reset() {
		super.reset();
		init();
	}
}
