package mosaic.util;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * The class LogPrinter provides facilities to easily write to one or more Logfiles via
 * static methods. It will always be appended to exising Files, but Methods
 * to delete one or all of them before writing to them are supported.
 *
 * The main method shows an example.
 *
 * The Logfile will be named:
 * baseName + "1"
 * baseName + "2"
 *
 * This is Thread safe, but blocking may occur during concurrent access.
 *
 * Please report bugs or suggestions to hilpold@swe.uni-linz.ac.at
 *
 * @author Thomas Hilpold
 * @version 15.07.1999
 **/
public class LogPrinter extends Object {

    private static Hashtable writers;

    public static String baseFileName = "JavaLog";
    public static String baseFileDir = "\\";

    static {
        writers = new Hashtable(10);
        //System.runFinalizersOnExit(true);
    }

    //
    // print methods for common use
    //

    /** prints a String to File 1 (one) */
    public static   void print(String p) {
        print(1, p);
    }

    /** prints a String to File 1 (one) adding a newline character */
    public static   void println(String p) {
        println(1, p);
    }

    /** prints a String to File logfile adding windows CR LF */
    public static   void println(int logFile, String p) {
        print(logFile, p + '\r' + '\n');
    }

    /** prints a String to File logfile */
    public static   void print(int logFile, String p) {
        FileWriter curWriter;

        if (logFile < 0) {
            System.err.println("No value " + logFile + " supported; must be >= 0");
            System.err.println("Could not print");
            return;
        }
        curWriter = getFileWriterForNr(logFile);
        try {
            curWriter.write(p);
            curWriter.flush();
        } catch (IOException e) {
            System.err.println(" Could not write to Logfile " + curWriter.toString());
        }
    }

    //
    // utility methods for special treatment
    //

    /** prints a String to File logfile */
    public static   boolean deleteLogFileNr(int logFileNr) {
        FileWriter openedFileWriter;
        File logFile;
        boolean deleteOK;
        String logFileName;

        if (logFileNr < 0) {
            System.err.println("No value " + logFileNr + " supported; must be >= 0");
            System.err.println("Could not delete");
            return false;
        }

        /* check if logFile is already open */
        if (writers.containsKey("" + logFileNr)) {
            openedFileWriter = (FileWriter)writers.get("" + logFileNr);
            try {
                openedFileWriter.close();
            } catch (IOException e) {
            }
            writers.remove("" + logFileNr);
            openedFileWriter = null;
        }
        logFileName = createLogFileNameForNr(logFileNr);
        logFile = new File(logFileName);
        deleteOK =logFile.delete();
        if (!deleteOK) {
            System.err.println("Could not delete or doesnt exist : " + logFile);
        }
        return deleteOK;
    }

    /** prints a String to File logfile */
    public static   boolean deleteAllLogFiles() {
        String []logFile;
        Enumeration allWriters;
        File curFile;
        boolean couldDeleteAll;
        int i;

        /* close all Writers */
        allWriters = writers.elements();
        while (allWriters.hasMoreElements()) {
            try {
                ((FileWriter)allWriters.nextElement()).close();
            } catch (IOException e) {
            }
        }
        writers = new Hashtable(10, 10);
        allWriters = null;

        /* find baseFileName*.* */
        logFile = (new File(baseFileDir)).list(
            new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return name.startsWith(baseFileName);
                }
            });
        i = 0;
        couldDeleteAll = true;
        System.out.println(" " + logFile);
        while ((i < logFile.length) && (couldDeleteAll)) {
            curFile = new File(baseFileDir + logFile[i]);
            couldDeleteAll = curFile.delete();
            System.out.println(" " + curFile);
            i++;
        }
        return couldDeleteAll;
    }

    //
    // internal helper methods
    //

    private static String createLogFileNameForNr(int logFile) {
        String key;
        String logFileName;

        key = "" + logFile;
        logFileName = baseFileDir + baseFileName + key + ".txt";
        return logFileName;
    }

    private static FileWriter getFileWriterForNr(int i) {
        /* looks for or if necessary creates a Filewriter for i */
        String key;
        String logFileName;
        FileWriter curWriter;

        key = "" + i;
        logFileName = createLogFileNameForNr(i);
        if (writers.containsKey(key)) {
            curWriter = (FileWriter)writers.get(key);
        } else {
            try {
                curWriter = new FileWriter(logFileName, true);
                writers.put(key, curWriter);
            } catch (IOException e) {
                curWriter = null;
                System.err.println("Could not open File " + baseFileName + key);
            }
        }
        return curWriter;
    }


    //
    // A test routine and example of use
    //

    /** performs a test */
    public static void main(String[] argv) {
        //
        // delete all Files on startup
        // (otherwise append occurs); rarely useful
        LogPrinter.deleteAllLogFiles();
        // open and print to file \JavaLog1.txt
        LogPrinter.print(1, "Printo File 1");
        LogPrinter.println(1, "ok ?, this is still the first line");
        LogPrinter.print("will print to file 1; default file");
        LogPrinter.println("    still same line");
        // open and print to file \JavaLog2.txt
        // open and print to file \JavaLog1.txt
        LogPrinter.print(2, "KJHDKJSHDKJSHD");
        LogPrinter.println(100123132, "KJHDKJSHDKJSHD");
        LogPrinter.println(2, "KJHDKJSHDKJSHD");
        LogPrinter.println(1, "KJHDKJSHDKJSHD");
        // open and print to file \JavaLog2.txt
        // create a new File
        LogPrinter.print(4, "KJHDKJSHDKJSHD");
        LogPrinter.print(5, "KJHDKJSHDKJSHD");
        LogPrinter.print(6, "6 will be deleted");
        LogPrinter.deleteLogFileNr(6);
        LogPrinter.println(6, "6");
        LogPrinter.println(6, "66");
        LogPrinter.println(6, "665");
        //
        // try to delete Logs, which dont exist
        //
        LogPrinter.deleteLogFileNr(4);
        LogPrinter.deleteLogFileNr(0);
        LogPrinter.deleteLogFileNr(-1);
        LogPrinter.deleteLogFileNr(1000);
        //
        // stop
        //
        System.err.println(" FINISHED PLEASE HIT CTRL - C - and lookup" +
                " your root dir");
        try {
            Thread.sleep(1000000);
        } catch (InterruptedException e) {
        }
    }
}
