package mosaic.util.custom;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 * Cell Editor which drags the values through the PropertyEditor for a
 * given Componenttype.
 *
 * Creates and uses a second Celleditor as Delegate.
 *
 * @version 17 Feb 1999
 * @author Thomas Hilpold
 * @see CustomCellEditor
 * @see CustomCellTextEditor
 */
public class CustomArrayCellEditor extends Object
    implements TableCellEditor, CellEditorListener {

    //
    // Data
    //

    protected PropertyEditor pe;
    protected TableCellEditor dce;
    protected Class componentType;
    protected EventListenerList listenerList = new EventListenerList();
    protected ChangeEvent changeEvent;

    //
    // For Debugging Output
    //

    private boolean[] DEBUG = new boolean[] {false, false, false};

    //
    // Construction
    //

    /**
     * Finds a Propertyeditor for the given Class and creates
     * an appropriate Celleditor, which will be used as Delegate.
     *
     * @param aComponentType Class, whose Objects are in the Cells
     */
    public CustomArrayCellEditor(Class aComponentType) {
        String [] searchPath;
        this.componentType = aComponentType;
        pe = PropertyEditorManager.findEditor(componentType);

        //*** Debugging code
        if (DEBUG[0]) {
            if (pe != null)
                OutFrame.out("We have a PE "+ pe.toString());
            else
                OutFrame.out("No PE for Type :"+ componentType.toString());
            searchPath = PropertyEditorManager.getEditorSearchPath();
            for (int i =0; i < searchPath.length; i++) {
                OutFrame.out("ManagerSearchpath = "+ searchPath[i]);
            }
        }
        //**** End of Debugging Code

        //determine type of PE
        if (pe == null) {
            //no propertyEditor -> Textfield and String Settings
            dce = new DefaultCellEditor(new JTextField());
        }
        else if (pe.supportsCustomEditor()) {
            dce = new CustomCellEditor(pe);
        }
        else if (pe.getTags() != null) {
            String[] tags = pe.getTags();
            JComboBox comboBox = new JComboBox();
            for (int i = 0; i < tags.length; i++)
                comboBox.addItem(tags[i]);
            dce = new DefaultCellEditor(comboBox);
       }
       else {
            dce = new CustomCellTextEditor(pe);
       }
       dce.addCellEditorListener(this);
       if (DEBUG[0]) OutFrame.out(this + " Cell Editor created");
    }

    /**
     * Forwarding Method Calls to the dce Object
     * which are implemented in Interface TableCellEditor
     * with passing the value through the PropertyEditor
     * if necessary.
     *
     * @return Object, which is an instance of ComponentType
     */
    public Object getCellEditorValue() {
        Object val = dce.getCellEditorValue();
        if (DEBUG[2]) System.err.println(this + " getCellEdValue");
        if (DEBUG[0]) OutFrame.out("CellEditor - getValue" + val);
        if (val == null)
            return null;
        else {
            if (DEBUG[0]) OutFrame.out ("CellEditor - i got  valClass =" + val.getClass().toString());
            if (pe == null) {
               //we get a String - because there is a textfield
            }
            else if (pe.supportsCustomEditor()) {
                //donothing
            }
            else if (pe.getTags() != null) {
                //we get a String - so...
                Object prev = pe.getValue();
                try {
                    pe.setAsText((String)val);
                } catch (IllegalArgumentException e) {
                    JOptionPane.showMessageDialog(null,
                            "Could not set Value\n"+ e.getMessage());

                        /**
                         * some editors do not like null to be set
                         * so if they already contain null
                         * we can avoid it
                         * otherwise the have to cope with it somhow
                         */
                        if (pe.getValue() != prev) pe.setValue(prev);
                };
                val = pe.getValue();
            }
            else {
                //try set and setastext
                Object prev = pe.getValue();
                if ((componentType.getClass().isInstance(val))
                    && (pe.getValue() != val))
                        pe.setValue(val);
                if (val instanceof String) {
                    try {
                        pe.setAsText((String)val);
                    } catch (IllegalArgumentException e) {
                        JOptionPane.showMessageDialog(null,
                                "Could not set Value\n"+ e.getMessage());
                        //some editors fail setting null
                        if (pe.getValue() != prev) pe.setValue(prev);
                    };
                }
                val = pe.getValue();
            }
            if (DEBUG[0]) OutFrame.out ("CellEditor - returned valClass =" + val.getClass().toString());
            return val;
        }
    }

    public boolean isCellEditable(EventObject anEvent) {
    	return dce.isCellEditable(anEvent);
	}

    public boolean shouldSelectCell(EventObject anEvent) {
    	return dce.shouldSelectCell(anEvent);
    }

    public boolean stopCellEditing() {
       	return dce.stopCellEditing();
    }

    public void cancelCellEditing() {
        dce.cancelCellEditing();
	}

    //
    //  Event Listeners
    //

    public void addCellEditorListener(CellEditorListener l) {
    	listenerList.add(CellEditorListener.class, l);
    }

    public void removeCellEditorListener(CellEditorListener l) {
	    listenerList.remove(CellEditorListener.class, l);
    }

    protected void fireEditingStopped() {
    if (DEBUG[2]) System.err.println(this + " FireEStopped not propagated");
	Object[] listeners = listenerList.getListenerList();
	for (int i = listeners.length-2; i>=0; i-=2) {
	    if (listeners[i]==CellEditorListener.class) {
		if (changeEvent == null)
		    changeEvent = new ChangeEvent(this);
		((CellEditorListener)listeners[i+1]).editingStopped(changeEvent);
	    }
	}
    }

    protected void fireEditingCanceled() {
        if (DEBUG[2]) System.err.println(this + " FireECanceled not propagated");
    	Object[] listeners = listenerList.getListenerList();
    	for (int i = listeners.length-2; i>=0; i-=2) {
    	    if (listeners[i]==CellEditorListener.class) {
    		if (changeEvent == null)
    		    changeEvent = new ChangeEvent(this);
    		((CellEditorListener)listeners[i+1]).editingCanceled(changeEvent);
    	    }
    	}
    }

    //
    //  Implementing the TableCellEditor Interface :
    //

    public Component getTableCellEditorComponent(JTable table, Object value,
						 boolean isSelected,
						 int row, int column) {
        if (DEBUG[0]) OutFrame.out("CellEd - get EditorComponent ");
        return dce.getTableCellEditorComponent(table, value, isSelected,
                			    		   row, column);
    }

    //
    // CellEditorListener Interface
    //

    public void editingCanceled(ChangeEvent e) {
        if (DEBUG[2]) System.err.println(this + " EditingCanceled not propagated");
        fireEditingCanceled();
    }

    public void editingStopped(ChangeEvent e) {
        if (DEBUG[2]) System.err.println(this + " EditingStopped not propagated");
        fireEditingStopped();
    }
}
