package mosaic.util.custom;

/**
 * Subclasses of this PropertyEditor will allow choosing and
 * customizing a Bean out of a given set of Subclasses using a
 * propertysheet.
 * <p>
 * Make sure the Name of the subclassed Editor is <PropertyType>+"Editor" and
 * return the set of Subclasses of <PropertyType> in the abstract method
 * getSubClasses() in a SubClass.
 * <p>
 * The String representation will be the classname of the currently set subclass
 * <p>
 * A CustomEditor can be opened, that supports a PropertySheet.
 * The value null is allowed to be set, because all possible Classes are known
 * here and therefore can be instantiated by calling their non-argument
 * constructor.
 * <p>
 *
 * @author Thomas Hilpold
 * @version 17 Feb 1999
 */
public abstract class MultiBeanEditor extends GenericBeanEditor {

    /** non - argument constructor */
    public MultiBeanEditor() {
        if (subClasses == null) {
            subClasses = getSubClasses();
            //assert subClasses != null
            customTags = new String[subClasses.length];
            for (int i = 0; i < subClasses.length; i++)
                if (subClasses[i] != null)
                    customTags[i] = subClasses[i].getName();
                else
                    customTags[i] = "null";
      }
    }

    /**
     * overwrite this method to support the subclasses to the class this
     * editor is intended to be used.
     * Null is an allowed component of the array and will be selectable
     * for the user. If null is not a component of the returned array
     * it can not be set by the user in the visual editor.
     */
    protected abstract Class[] getSubClasses();

    /** returns only the ClassName */
    public String getAsText () {
        if (DEBUG[0]) OutFrame.out(this.toString() + " GETASTEXT ");
        Object val = super.getValue();
        if (val != null)
            return val.getClass().getName();
        else
            return "null";
    }

    /**
     * tries to set new Objects using their non-argument contructors
     * if the given String is not found in the Tags, an IllegalArgumentExceotion
     * will be thrown
     */
    public void setAsText(String text)
            throws java.lang.IllegalArgumentException {
        if (DEBUG[0]) OutFrame.out(this.toString() + " SETASTEXT: " + text);
        boolean found = false;
        int i = 0;
        while (! found && i < subClasses.length) {
        	if (customTags[i].equals(text))
            	found = true;
            else
	            i++;
        }
    	if (found) {
            if (customTags[i].equals("null"))
                setValue(null);
            else {
            	try {
    				setValue((subClasses[i]).newInstance());
                }
                catch (IllegalAccessException e) {
                	setValue (null);
                    throw (new IllegalArgumentException(
                        "Could not access Constructor of class "
                        + customTags[i] + " !"));
                }
                catch (InstantiationException e) {
                	setValue (null);
                    throw (new IllegalArgumentException(
                        "InstantiatianException of class "
                        + customTags[i] + "!"));
                }
            }
        }
        else
            throw new IllegalArgumentException("String \"" + text + "\" not recognized !");
    }

    //
    // Implementing interface CustomTagsPropertyEditor
    //

    /** implements interface CustomTagsPropertyEditor */
    public boolean supportsCustomTags() {
        return true;
    }

    /**
     * implements interface CustomTagsPropertyEditor
     * @return classnames of possible subclasses
     **/
    public String[] getCustomTags() {
        return customTags;
    }

    //
    // Data
    //

    private static Class[] subClasses;
    private static String[] customTags;
}
