package mosaic.util.custom;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;

/**
 * Instances contain a propertysheet and, if the CustomTagsPropertyeditor
 * supports tags, also a AWT - choice which displays those tags.
 *
 * It should be used as a visual editor returned by a PropertyEditors
 * getCustomEditor Method.
 *
 * Current implementation uses Choice from AWT and SWING JPanel
 *
 *  First release date:
 *  09 Jan 1999
 *
 *  Changes:
 *  27 Apr 1999 Hilpold | Fix for bug: when JBuilder 2.0.1 embedded this JPanel in his
 *                      | own Inspector Dialog, the propertysheet became invisible each time
 *                      | a user selected another item from the choice
 *
 *  28 Apr 1999 Hilpold | A MessageDialog is shown, if setAsText() on the PropertyEditor
 *                      | fails, due to implementation faults of the PropertyEditor
 *                      | and the choice resets to right selection
 *
 *  @author Thomas Hilpold
 *  @version 28 Apr 1999
 */

public class PropertyCustomEditor extends JPanel
    implements ItemListener, PropertySheetListener {

    /**
     *  The CustomTagsPropertyEditor will be used to get the bean
     *  (getValue()) copy it, and apply changes (setValue()) setting
     *  the cloned object.
     */
    public PropertyCustomEditor(CustomTagsPropertyEditor aPE) {
        super(false);                                     //no double buffering
        propEditor = aPE;
        hasTags = propEditor.supportsCustomTags();
        if (hasTags)
          propTags = propEditor.getCustomTags();
        else
          propTags = null;
        propSheet = new PropertySheet(aPE.getValue(), this);
        initBasic();
        if (hasTags)
            initWithTags();
    }

    private void initBasic() {
    	setLayout(new BorderLayout());
		setMinimumSize(new Dimension(250,270));
		setPreferredSize(new Dimension(280,270));
	    propSheetPanel = propSheet.getScrollPane();
        add(propSheetPanel, "Center");
	}

    private void initWithTags() {
        objectChoice = new Choice();
		add(objectChoice, "North");
		for (int i = 0; i < propTags.length; i++) {
		    objectChoice.addItem(propTags[i]);
		}
     	objectChoice.select(propEditor.getAsText()); //AWT version
     	objectChoice.addItemListener(this);
    }

    //
    // ItemListener Methods
    //

    /**
     * even if the selection in the choice is the same item as before,
     * a new bean will be created calling the no argument constructor
     */
    public void itemStateChanged(ItemEvent e) {
       Object currentBean;
       Object source = e.getSource();
       if (source == objectChoice) {
            String choice = (String)objectChoice.getSelectedItem();
            try {
                propEditor.setAsText(choice);
            } catch (IllegalArgumentException ie) {
                JOptionPane.showMessageDialog(null,
                    "IllegalArgumentException when calling setAsText( " + choice + ") \n"+
                    " on Property Editor " + propEditor + "\n" +
                    ((ie.getMessage() != null)? ie.getMessage():""));
                  	objectChoice.select(propEditor.getAsText()); //AWT version
            };
            currentBean = propEditor.getValue();
            remove(propSheetPanel);
            propSheet.beanChanged(currentBean);
            propSheetPanel = propSheet.getScrollPane();
            add(propSheetPanel, "Center");

            /*
            * workaround for JBuilder 2-bug in Design mode
            * new propSheetPanel now visible after the choices pop-up pops down
            */
            getTopLevelAncestor().validate();
            //repaint();
       }
    }

    /**
     *  looks up the propertysheet and applies its current object to
     *  the propertyeditor.
     */
    public void fireChanges(String propName) {
      if (propSheet != null)
          propEditor.setValue(propSheet.getBean());
    }

    //
    // Data
    //

    private boolean hasTags;
    protected CustomTagsPropertyEditor propEditor;
    protected String[] propTags;
    protected PropertySheet propSheet;
    protected JScrollPane propSheetPanel;
	protected Choice objectChoice;
}
