//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------
// class TextInputView    ------------------------------------------------------
// -----------------------------------------------------------------------------

package mosaic.vis.input;

import mosaic.sim.kernel.variable.*;
import mosaic.sim.kernel.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* Animation view to visualize textual representations of arbitrary values.
* Gives out the value as its textual representation using <code>toString</code>.
* Allows to enter a new string, which is forwarded to a connected UserInputListener
* by pressing <code> RETURN <code>
* <p>
* Use the view by adding it as VariableChangeListener to a variable of arbitrary type .
* Use the input functionality by registering a userInputListener at this.
* <p>
* Costumization: 
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*/


public abstract class TextInputView extends VariableInputView {
// Textfield allows entering a text.
JTextField textField = new JTextField();

  /** Default constructor for TextInputView */
  public TextInputView() {
    text = "";
		try  {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
  }

  private void jbInit() throws Exception {
		textField.setText(text);
		textField.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				textField_actionPerformed(e);
			}
		});
		textField.addFocusListener(new java.awt.event.FocusAdapter() {
			public void focusLost(FocusEvent e) {
				textField_focusLost(e);
			}
			public void focusGained(FocusEvent e) {
				textField_focusGained(e);
			}
		});

		textField.setBounds(new Rectangle(0, 0, 160, 34));
		this.add(textField, null);
	}

  public void setBounds(int x, int y, int w, int h)
  { super.setBounds(x,y,w,h);
    textField.setBounds(new Rectangle(0,0,w,h));
  }

  public void setBounds(Rectangle r)
  { this.setBounds(r.x, r.y, r.width, r.height);
  }

  public Dimension getPreferredSize()
  { return (new Dimension(50,20));
  }

  public Dimension getMinimumSize() {
    return (new Dimension(30,10));
  }

//  text output ----------------------------------------------------------------

  /** variable to hold the textual representation of the variable value*/
  protected String text;

  /** Sets the textual representation of the variable value*/
  public void setText(String newText) {
    text = newText;
    textField.setText(text);
  }

  /** Gets the textual representation of the variable value*/
  public String getText() {
    return text;
  }
// -----------------------------------------------------------------------------
// Access to textField

    public JTextField getTextField( ) {
        return textField;
    }


// setValue --------------------------------------------------------------------

  /** 
  * Specialized method to set a new value; 
  * also sets the textual represenation using <code>toString</code>. 
  */
  public void setValue (Object newVal) {
      //super.setValue(newVal);
      value = newVal;
      String htext = "";
      if (getValue() != null) 
          htext = htext + getValue().toString();
      setText (htext);
    }
// -----------------------------------------------------------------------------
// interface VariableChangeListener
  /**
  * Reacts to a variableChange event by setting the value and repainting the view,
  * only if focus is not gained (no user input)
  */
  public void variableChange(VariableChangeEvent e) {
      if (!focusGained) {
        super.variableChange(e);
//        paintCached();

      }
  }

// -----------------------------------------------------------------------------
// Properties

  /** Defines background color if focus gained. */
  Color highlightColor = Color.yellow;

  public Color getHighlightColor()
  { return highlightColor;
  }

  public void setHighlightColor(Color c)
  { highlightColor = c;
  }

  /** Store backgroundColor to enable reset if focus lost. */
  Color backgroundColor;

  /** used to check if actually focus gained or lost */
  boolean focusGained = false;

  /** sets tooltipText of textField */
  public void setToolTipText(String txt) {
      textField.setToolTipText(txt);
      super.setToolTipText(txt);
  }
  
  /** Gets called if focus is gained. Highlight background in previous defined
      color.
  */
  void textField_focusGained(FocusEvent e) {
   backgroundColor = textField.getBackground();
   textField.setBackground(highlightColor);
   textField.select(0, getText().length());
   focusGained=true;
  }

  /** Reset backgroundcolor in inputfield if focus lost, and reset text in
      inputField to old value, any userInput is ignored !
  */
  	void textField_focusLost(FocusEvent e) {
    textField.setBackground(backgroundColor);
    textField.setText(getText());  // reset to old value
    focusGained=false;
  }

  /** Gets called if user types RETURN in textField.
      @param e holds ActionEvent
  */
	void textField_actionPerformed(ActionEvent e) {
    Object valObj;

    if ((AWTEvent.KEY_EVENT_MASK & e.getID()) != 0){
      valObj = readValue(textField.getText());
      if (valObj != null)
      { setValue(valObj);
        triggerEvent();
      }
      textField.setText(text);   // always set text in text field to actual text
    }                            // if entered value was a correct but not accepted
  }                              // by another reason, also reset text.

  /** Implement in derived class to parse special values from String
      returns Object containing extracted value if successful.
      else returns null.
  */
  abstract Object readValue(String text);

// paint -----------------------------------------------------------------------

  /**
  * Paints the object;
  * by giving out its textual representation.
  *
  * @param g the graphics context of the view
  */
  public void paintValue(Graphics g) {
    if (!focusGained){
      g.setColor(getForeground());
      textField.setText(getText());
    }
  }

// setFont ---------------------------------------------------------------------
    public void setFont(Font f)
    {   textField.setFont(f);
    }

// setFont ---------------------------------------------------------------------
    public void setBackground(Color c)
    {   if (textField != null)
            textField.setBackground(c);
    }

}
      
